from axelrod import Player
import random


class AverageCopier(Player):
    """The player will cooperate with probability p if the opponent's cooperation ratio is p."""

    name = 'Average Copier'

    def strategy(self, opponent):
        """Randomly picks a strategy (not affected by history)."""
        if len(opponent.history) == 0:
            return random.choice(['C', 'D'])
        p = sum([s == 'C' for s in opponent.history]) / len(opponent.history)
        rnd_num = random.random()
        if rnd_num < p:
            return 'C'
        return 'D'


class NiceAverageCopier(Player):
    """Same as Average Copiero, but always starts by cooperating."""

    name = 'Nice Average Copier'

    def strategy(self, opponent):
        """Randomly picks a strategy (not affected by history)."""
        if len(opponent.history) == 0:
            return 'C'
        p = sum([s == 'C' for s in opponent.history]) / len(opponent.history)
        rnd_num = random.random()
        if rnd_num < p:
            return 'C'
        return 'D'


from axelrod import Player


class GoByMajority(Player):
    """A player examines the history of the opponent: if the opponent has more defections than cooperations then the player defects.

    An optinal memory attribute will limit the number of turns remembered
    """

    memory = 0

    def strategy(self, opponent):
        """This is affected by the history of the opponent.

        As long as the opponent cooperates at least as often as they defect then the player will cooperate.
        If at any point the opponent has more defections than cooperations in memory the player defects.
        """
        history = opponent.history[-self.memory:]
        if sum([s == 'D' for s in history]) > sum([s == 'C' for s in history]):
            return 'D'
        return 'C'

    def __repr__(self):
        """The string method for the strategy."""
        return 'Go By Majority' + (self.memory > 0) * ("/%i" % self.memory)


class GoByMajority40(GoByMajority):
    memory = 40

class GoByMajority20(GoByMajority):
    memory = 20

class GoByMajority10(GoByMajority):
    memory = 10

class GoByMajority5(GoByMajority):
    memory = 5




import math
import csv

from StringIO import StringIO


def median(lst):
    lst = sorted(lst)
    if len(lst) < 1:
            return None
    if len(lst) %2 == 1:
            return lst[((len(lst)+1)/2)-1]
    if len(lst) %2 == 0:
            return float(sum(lst[(len(lst)/2)-1:(len(lst)/2)+1]))/2.0


class ResultSet(object):
    """A class to hold the results of a tournament."""

    def __init__(self, players, turns, repetitions):

        self.players = players
        self.nplayers = len(players)

        self.turns = turns
        self.repetitions = repetitions

        plist = list(range(self.nplayers))
        replist = list(range(repetitions))
        self.results = [[[0 for r in replist ] for j in plist] for i in plist]

        self.finalised = False

    def generate_scores(self):
        """Return normalized scores based on the results.

        Originally there were no self-interactions, so the code here was rewritten
        to exclude those from the generated score. To include self-interactions,
        remove the condition on ip and ires and fix the normalization factor.
        """

        scores = []
        for ires, res in enumerate(self.results):
            scores.append([])
            for irep in range(self.repetitions):
                scores[-1].append(0)
                for ip in range(self.nplayers):
                    if ip != ires:
                        scores[-1][-1] += res[ip][irep]

        normalization = self.turns * (self.nplayers - 1)
        scores_normalized = [[1.0 * s / normalization for s in r] for r in scores]

        return scores_normalized

    def generate_ranking(self, scores):
        """
        Returns a list of players (their index within the
        players list rather than a player instance)
        ordered by median score
        """
        ranking = sorted(
            range(self.nplayers),
            key=lambda i: median(scores[i]))
        return ranking

    def generate_ranked_names(self, ranking):
        """Returns a list of players names sorted by their ranked order."""
        ranked_names = [str(self.players[i]) for i in ranking]
        return ranked_names

    def generate_payoff_matrix(self):
        """Returns a per-turn averaged payoff matrix and its stddevs."""
        averages = []
        stddevs = []
        for res in self.results:
            averages.append([])
            stddevs.append([])
            for s in res:
                perturn = [1.0 * rep / self.turns for rep in s]
                avg = sum(perturn) / self.repetitions
                dev = math.sqrt(sum([(avg - pt)**2 for pt in perturn]) / self.repetitions)
                averages[-1].append(avg)
                stddevs[-1].append(dev)
        return averages, stddevs

    def finalise(self, payoffs_list):
        if not self.finalised:
            for index, payoffs in enumerate(payoffs_list):
                for i in range(len(self.players)):
                    for j in range(len(self.players)):
                        self.results[i][j][index] = payoffs[i][j]
            self.scores = self.generate_scores()
            self.ranking = self.generate_ranking(self.scores)
            self.ranked_names = self.generate_ranked_names(self.ranking)
            self.payoff_matrix, self.payoff_stddevs = self.generate_payoff_matrix()
            self.finalised = True
        else:
            raise AttributeError('Result set is already finalised')

    def csv(self):
        if self.finalised:
            csv_string = StringIO()
            header = ",".join(self.ranked_names) + "\n"
            csv_string.write(header)
            writer = csv.writer(csv_string, lineterminator="\n")
            for irep in range(self.repetitions):
                data = [self.scores[rank][irep] for rank in self.ranking]
                writer.writerow(map(str, data))
            return csv_string.getvalue()
        else:
            raise AttributeError("Result Set is not finalised")



"""Test for the random strategy."""

import random

import axelrod

from test_player import TestPlayer


class TestRandom(TestPlayer):

    name = "Random"
    player = axelrod.Random
    stochastic = True

    def test_strategy(self):
        """Test that strategy is randomly picked (not affected by history)."""
        random.seed(1)
        P1 = axelrod.Random()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')
        random.seed(1)
        P1.history = ['C', 'D', 'C']
        P2.history = ['C', 'C', 'D']
        self.assertEqual(P1.strategy(P2), 'C')
        random.seed(2)
        self.assertEqual(P1.strategy(P2), 'D')



from axelrod import Player

class Grumpy(Player):
    """A player that defects after a ceratin level of grumpiness. Grumpiness increases when the opponent defects and decreases when the opponent co-operates."""

    name = 'Grumpy'

    def __init__(self, starting_state = 'Nice', grumpy_threshold = 10, nice_threshold = -10):
        """Player starts of nice be default with set thresholds"""
        super(Grumpy, self).__init__()
        self.history = []
        self.score = 0
        self.state = starting_state
        self.starting_state = starting_state
        self.grumpy_threshold = grumpy_threshold
        self.nice_threshold = nice_threshold

    def strategy(self, opponent):
        """A player that gets grumpier the more the opposition defects, and nicer the more they cooperate.

        Starts off Nice, but becomes grumpy once the grumpiness threshold is hit.
        Won't become nice once that grumpy threshold is hit, but must reach a much lower threshold before it becomes nice again.
        """

        self.grumpiness = sum(play=='D' for play in opponent.history) - sum(play=='C' for play in opponent.history)

        if self.state == 'Nice':
            if self.grumpiness > self.grumpy_threshold:
                self.state = 'Grumpy'
                return 'D'
            return 'C'

        if self.state == 'Grumpy':
            if self.grumpiness < self.nice_threshold:
                self.state = 'Nice'
                return 'C'
            return 'D'

    def reset(self):
        """Resets score, history and state for the next round of the tournement."""

        self.history = []
        self.state = self.starting_state



"""Test for the memoryone strategies."""

import random

import axelrod

from test_player import TestPlayer


class TestWinStayLostShift(TestPlayer):

    name = "Win-Stay Lose-Shift"
    player = axelrod.WinStayLoseShift

    def test_strategy(self):
        """Starts by cooperating"""
        P1 = self.player()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_effect_of_strategy(self):
        """Check that switches if does not get best payoff."""
        P1 = self.player()
        P2 = axelrod.Player()
        P1.history = ['C']
        P2.history = ['C']
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history = ['C']
        P2.history = ['D']
        self.assertEqual(P1.strategy(P2), 'D')
        P1.history = ['D']
        P2.history = ['C']
        self.assertEqual(P1.strategy(P2), 'D')
        P1.history = ['D']
        P2.history = ['D']
        self.assertEqual(P1.strategy(P2), 'C')

class TestGTFT(TestPlayer):

    name = "Generous Tit-For-Tat"
    player = axelrod.GTFT
    stochastic = True

    def test_strategy(self):
        P1 = self.player()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

        P1.history = ['C']
        P2.history = ['C']
        random.seed(2)
        # With probability .05 will defect
        self.assertEqual(P1.strategy(P2), 'D')
        # But otherwise will cooperate
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')

        P1.history = ['C']
        P2.history = ['D']
        random.seed(31)
        # With probability .05 will cooperate
        self.assertEqual(P1.strategy(P2), 'C')
        # But otherwise will defect
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')

        P1.history = ['D']
        P2.history = ['C']
        random.seed(2)
        # With probability .05 will defect
        self.assertEqual(P1.strategy(P2), 'D')
        # But otherwise will cooperate
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')

        P1.history = ['D']
        P2.history = ['D']
        random.seed(31)
        # With probability .05 will cooperate
        self.assertEqual(P1.strategy(P2), 'C')
        # But otherwise will defect
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')


class TestStochasticCooperator(TestPlayer):

    name = "Stochastic Cooperator"
    player = axelrod.StochasticCooperator
    stochastic = True

    def test_strategy(self):
        P1 = self.player()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

        P1.history = ['C']
        P2.history = ['C']
        random.seed(15)
        # With probability .065 will defect
        self.assertEqual(P1.strategy(P2), 'D')
        # But otherwise will cooperate
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')

        P1.history = ['C']
        P2.history = ['D']
        random.seed(1)
        # With probability .229 will cooperate
        self.assertEqual(P1.strategy(P2), 'C')
        # But otherwise will defect
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')

        P1.history = ['D']
        P2.history = ['C']
        random.seed(3)
        # With probability .266 will cooperate
        self.assertEqual(P1.strategy(P2), 'C')
        # But otherwise will defect
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')

        P1.history = ['D']
        P2.history = ['D']
        random.seed(13)
        # With probability .42 will cooperate
        self.assertEqual(P1.strategy(P2), 'C')
        # But otherwise will defect
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')

class TestStochasticWSLS(TestPlayer):

    name = "Stochastic WSLS"
    player = axelrod.StochasticWSLS
    stochastic = True

    def test_strategy(self):
        P1 = self.player()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

        P1.history = ['C']
        P2.history = ['C']
        random.seed(2)
        # With probability .05 will defect
        self.assertEqual(P1.strategy(P2), 'D')
        # But otherwise will cooperate
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')

        P1.history = ['C']
        P2.history = ['D']
        random.seed(31)
        # With probability .05 will cooperate
        self.assertEqual(P1.strategy(P2), 'C')
        # But otherwise will defect
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')

        P1.history = ['D']
        P2.history = ['C']
        random.seed(31)
        # With probability .05 will cooperate
        self.assertEqual(P1.strategy(P2), 'C')
        # But otherwise will defect
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')

        P1.history = ['D']
        P2.history = ['D']
        random.seed(2)
        # With probability .05 will defect
        self.assertEqual(P1.strategy(P2), 'D')
        # But otherwise will defect
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')



class Game(object):
    """A class to hold the game matrix and to score a game accordingly."""

    def __init__(self, r=2, s=0, t=5, p=4):
        self.scores = {
            ('C', 'C'): (r, r),
            ('D', 'D'): (p, p),
            ('C', 'D'): (t, s),
            ('D', 'C'): (s, t),
        }

    def RPTS(self):
        """Return the values in the game matrix in the Press and Dyson notation."""
        R = self.scores[('C', 'C')][0]
        P = self.scores[('D', 'D')][0]
        T = self.scores[('C', 'D')][0]
        S = self.scores[('D', 'C')][0]
        return (R, P, T, S)

    def score(self, pair):
        """Return the appropriate score for decision pair.

        Returns the appropriate score (as a tuple) from the scores dictionary
        for a given pair of plays (passed in as a tuple).
        e.g. score(('C', 'C')) returns (2, 2)
        """
        return self.scores[pair]



import unittest

import axelrod


class TestPlayer(unittest.TestCase):

    name = "Player"
    player = axelrod.Player
    stochastic = False

    def test_initialisation(self):
        """Test that the player initiates correctly."""
        self.assertEqual(self.player().history, [])
        self.assertEqual(self.player().stochastic, self.stochastic)

    def test_repr(self):
        """Test that the representation is correct."""
        self.assertEquals(str(self.player()), self.name)

    def test_strategy(self):
        """Test that strategy method."""
        self.assertEquals(self.player().strategy(self.player()), None)

    def test_reset(self):
        """Make sure reseting works correctly."""
        p = self.player()
        p.history = ['C', 'C']
        p.reset()
        self.assertEquals(p.history, [])



"""Test for the grumpy strategy."""

import axelrod

from test_player import TestPlayer


class TestGrumpy(TestPlayer):

    name = "Grumpy"
    player = axelrod.Grumpy
    stochastic = False

    def test_initial_nice_strategy(self):
        """
        Starts by cooperating
        """
        P1 = axelrod.Grumpy()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_initial_grumpy_strategy(self):
        """
        Starts by defecting if grumpy
        """
        P1 = axelrod.Grumpy(starting_state = 'Grumpy')
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'D')

    def test_strategy(self):
        """
        Tests that grumpy will play c until threshold is ht at which point it will become grumpy.
        Player will then not become nice until lower nice threshold is hit.
        """
        P1 = axelrod.Grumpy(grumpy_threshold = 3, nice_threshold=0)
        P2 = axelrod.Player()
        P1.history = ['C', 'D', 'D', 'D']
        P2.history = ['C', 'C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history = ['C', 'C', 'D', 'D', 'D']
        P2.history = ['D', 'D', 'D', 'D', 'D']
        self.assertEqual(P1.strategy(P2), 'D')
        P1.history = ['C', 'C', 'D', 'D', 'D', 'D', 'D', 'D']
        P2.history = ['D', 'D', 'D', 'D', 'D', 'C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'D')
        P1.history = ['C', 'C', 'D', 'D', 'D', 'D', 'D', 'D', 'D', 'D', 'D']
        P2.history = ['D', 'D', 'D', 'D', 'D', 'C', 'C', 'C', 'C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'C')

    def test_reset_method(self):
        """
        tests the reset method
        """
        P1 = axelrod.Grumpy(starting_state = 'Grumpy')
        P1.history = ['C', 'D', 'D', 'D']
        P1.state = 'Nice'
        P1.reset()
        self.assertEqual(P1.history, [])
        self.assertEqual(P1.state, 'Grumpy')



"""Test for the tit for tat strategies."""

import axelrod

from test_player import TestPlayer


class TestTitForTat(TestPlayer):

    name = "Tit For Tat"
    player = axelrod.TitForTat

    def test_strategy(self):
        """Starts by cooperating."""
        P1 = axelrod.TitForTat()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_effect_of_strategy(self):
        """Repeats last action of opponent history."""
        P1 = axelrod.TitForTat()
        P2 = axelrod.Player()
        P2.history = ['C', 'C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'C')
        P2.history = ['C', 'C', 'C', 'C', 'D']
        self.assertEqual(P1.strategy(P2), 'D')


class TestTitFor2Tats(TestPlayer):

    name = 'Tit For 2 Tats'
    player = axelrod.TitFor2Tats

    def test_strategy(self):
        """Starts by cooperating."""
        P1 = axelrod.TitFor2Tats()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_effect_of_strategy(self):
        """Will defect only when last two turns of opponent were defections."""
        P1 = axelrod.TitFor2Tats()
        P2 = axelrod.Player()
        P1.history = ['C', 'C', 'C']
        P2.history = ['C', 'D', 'D']
        self.assertEqual(P1.strategy(P2), 'D')
        P1.history = ['C', 'C', 'D', 'D']
        P2.history = ['D', 'D', 'D', 'C']
        self.assertEqual(P1.strategy(P2), 'C')


class TestTwoTitsForTat(TestPlayer):

    name = 'Two Tits For Tat'
    player = axelrod.TwoTitsForTat

    def test_strategy(self):
        """
        Starts by cooperating
        """
        P1 = axelrod.TwoTitsForTat()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_effect_of_strategy(self):
        """
        Will defect only when last two turns of opponent were defections.
        """
        P1 = axelrod.TwoTitsForTat()
        P2 = axelrod.Player()
        P1.history = ['C', 'C']
        P2.history = ['D', 'D']
        self.assertEqual(P1.strategy(P2), 'D')
        P1.history = ['C', 'C', 'D']
        P2.history = ['D', 'D', 'C']
        self.assertEqual(P1.strategy(P2), 'D')
        P1.history = ['C', 'C', 'D', 'D']
        P2.history = ['D', 'D', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'C')


class TestSuspiciousTitForTat(TestPlayer):

    name = "Suspicious Tit For Tat"
    player = axelrod.SuspiciousTitForTat

    def test_strategy(self):
        """Starts by defecting"""
        P1 = axelrod.SuspiciousTitForTat()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'D')

    def test_affect_of_strategy(self):
        """Will do opposite of what opponent does."""
        P1 = axelrod.SuspiciousTitForTat()
        P2 = axelrod.Player()
        P1.history = ['D']
        P2.history = ['C']
        self.assertEqual(P1.strategy(P2), 'D')
        P1.history.append('D')
        P2.history.append('D')
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history.append('C')
        P2.history.append('D')
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history.append('C')
        P2.history.append('C')
        self.assertEqual(P1.strategy(P2), 'D')


class TestSneakyTitForTat(TestPlayer):

    name = "Sneaky Tit For Tat"
    player = axelrod.SneakyTitForTat

    def test_strategy(self):
        """Starts by cooperating."""
        P1 = axelrod.SneakyTitForTat()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_effect_of_strategy(self):
        """Will try defecting after two turns of cooperation, but will stop if punished."""
        P1 = axelrod.SneakyTitForTat()
        P2 = axelrod.Player()
        P1.history = ['C', 'C']
        P2.history = ['C', 'C']
        self.assertEqual(P1.strategy(P2), 'D')
        P1.history = ['C', 'C', 'D', 'D']
        P2.history = ['C', 'C', 'C', 'D']
        self.assertEqual(P1.strategy(P2), 'C')


import random

from axelrod import Player


class MetaPlayer(Player):
    """A generic player that has its own team of players."""

    team = []

    def __init__(self):

        Player.__init__(self)

        # Make sure we don't use any meta players to avoid infinite recursion.
        self.team = [t for t in self.team if not issubclass(t, MetaPlayer)]
        self.nteam = len(self.team)

        # Initiate all the player in out team.
        self.team = [t() for t in self.team]

        # If the team will have stochastic players, this meta is also stochastic.
        self.stochastic = any([t.stochastic for t in self.team])

    def strategy(self, opponent):

        # Make sure the history of all hunters is current.
        for ih in range(len(self.team)):
            self.team[ih].history = self.history

        # Get the results of all our players.
        results = [player.strategy(opponent) for player in self.team]

        # A subclass should just define a way to choose the result based on team results.
        return self.meta_strategy(results)

    def meta_strategy(self, results):
        """Determine the meta result based on results of all players."""
        pass


class MetaMajority(MetaPlayer):
    """A player who goes by the majority vote of all other non-meta players."""

    name = "Meta Majority"

    def __init__(self):

        # We need to import the list of strategies at runtime, since
        # _strategies import also _this_ module before defining the list.
        from _strategies import ordinary_strategies
        self.team = ordinary_strategies

        MetaPlayer.__init__(self)

    def meta_strategy(self, results):
        if results.count('D') > results.count('C'):
            return 'D'
        return 'C'


class MetaMinority(MetaPlayer):
    """A player who goes by the minority vote of all other non-meta players."""

    name = "Meta Minority"

    def __init__(self):

        # We need to import the list of strategies at runtime, since
        # _strategies import also _this_ module before defining the list.
        from _strategies import ordinary_strategies
        self.team = ordinary_strategies

        MetaPlayer.__init__(self)

    def meta_strategy(self, results):
        if results.count('D') < results.count('C'):
            return 'D'
        return 'C'


class MetaWinner(MetaPlayer):
    """A player who goes by the strategy of the current winner."""

    name = "Meta Winner"

    def __init__(self, team=None):

        # The default is to used all strategies available, but we need to import the list
        # at runtime, since _strategies import also _this_ module before defining the list.
        if team:
            self.team = team
        else:
            from _strategies import ordinary_strategies
            self.team = ordinary_strategies

        MetaPlayer.__init__(self)

        # For each player, we will keep the history of proposed moves and
        # a running score since the beginning of the game.
        for t in self.team:
            t.proposed_history = []
            t.score = 0

    def strategy(self, opponent):

        # Update the running score for each player, before determining the next move.
        if len(self.history):
            for player in self.team:
                pl_C = player.proposed_history[-1] == "C"
                opp_C = opponent.history[-1] == "C"
                s = 2 * (pl_C and opp_C) or 5 * (pl_C and not opp_C) or 4 * (not pl_C and not opp_C) or 0
                player.score += s

        return MetaPlayer.strategy(self, opponent)

    def meta_strategy(self, results):

        scores = [pl.score for pl in self.team]
        bestscore = min(scores)
        beststrategies = [i for i, pl in enumerate(self.team) if pl.score == bestscore]
        bestproposals = [results[i] for i in beststrategies]
        bestresult = "C" if "C" in bestproposals else "D"

        # Update each player's proposed history with his proposed result, but always after
        # the new result has been settled based on scores accumulated until now.
        for r, t in zip(results, self.team):
            t.proposed_history.append(r)

        return bestresult


from axelrod import Player
import random

class Inverse(Player):
    """A player who defects with a probability that diminishes relative to how long ago the opponent defected."""

    name = 'Inverse'

    def strategy(self, opponent):
        """Looks at opponent history to see if they have defected.

        If so, player defection is inversely proportional to when this occurred.
        """

        index = next((index for index,value in enumerate(opponent.history, start = 1) if value == 'D'), None)

        if index == None:
            return 'C'

        rnd_num = random.random()

        if rnd_num < 1/abs(index):
            return 'D'
        return 'C'



"""Test for the punisher strategy."""

import random

import axelrod

from test_player import TestPlayer


class TestPunisher(TestPlayer):

    name = "Punisher"
    player = axelrod.Punisher
    stochastic = False

    def test_init(self):
        """Tests for the __init__ method."""
        P1 = axelrod.Punisher()
        self.assertEqual(P1.history, [])
        self.assertEqual(P1.score, 0)
        self.assertEqual(P1.mem_length, 1)
        self.assertEqual(P1.grudged, False)
        self.assertEqual(P1.grudge_memory, 1)

    def test_strategy(self):
        """Starts by cooperating."""

        random.seed(4)

        P1 = axelrod.Punisher()
        P2 = axelrod.Player()
        P2.history = ['C', 'C', 'D', 'D', 'D', 'C']

        self.assertEqual(P1.mem_length, 1)
        self.assertEqual(P1.grudged, False)

        # Starts by playing C
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('C')

        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('D')

        self.assertEqual(P2.history, ['C', 'C', 'D', 'D', 'D', 'C', 'C', 'D'])
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.grudged, True)

        for turn in range(P1.mem_length-1):
            self.assertEqual(P1.mem_length, 10)
            self.assertEqual(P1.strategy(P2), 'D')
            # Doesn't matter what opponent plays now
            P2.history.append(random.choice(['C', 'D']))
            self.assertEqual(P1.grudged, True)

        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.grudged, True)
        P2.history.append('C')

        # Back to being not grudged
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('C')

        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('D')

        # Now grudges again
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.mem_length, 8)
        self.assertEqual(P1.grudged, True)
        P2.history.append('C')

        for turn in range(P1.mem_length-1):
            self.assertEqual(P1.mem_length, 8)
            self.assertEqual(P1.strategy(P2), 'D')
            # Doesn't matter what opponent plays now
            P2.history.append(random.choice(['C', 'D']))
            self.assertEqual(P1.grudged, True)

        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.grudged, True)
        P2.history.append('C')

        # Back to being not grudged
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('C')

    def test_reset_method(self):
        """Tests the reset method."""
        P1 = axelrod.Punisher()
        P1.history = ['C', 'D', 'D', 'D']
        P1.grudged = True
        P1.grudge_memory = 4
        P1.reset()
        self.assertEqual(P1.history, [])
        self.assertEqual(P1.grudged, False)
        self.assertEqual(P1.grudge_memory, 0)


class TestInversePunisher(TestPlayer):

    name = "Inverse Punisher"
    player = axelrod.InversePunisher
    stochastic = False

    def test_init(self):
        """Tests for the __init__ method."""
        P1 = axelrod.InversePunisher()
        self.assertEqual(P1.history, [])
        self.assertEqual(P1.score, 0)
        self.assertEqual(P1.mem_length, 1)
        self.assertEqual(P1.grudged, False)
        self.assertEqual(P1.grudge_memory, 1)


    def test_strategy(self):
        """Starts by cooperating."""

        random.seed(4)

        P1 = axelrod.InversePunisher()
        P2 = axelrod.Player()
        P2.history = ['C', 'C', 'D', 'D', 'D', 'C']

        self.assertEqual(P1.mem_length, 1)
        self.assertEqual(P1.grudged, False)

        # Starts by playing C
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('C')

        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('D')

        self.assertEqual(P2.history, ['C', 'C', 'D', 'D', 'D', 'C', 'C', 'D'])
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.grudged, True)

        for turn in range(P1.mem_length-1):
            self.assertEqual(P1.mem_length, 10)
            self.assertEqual(P1.strategy(P2), 'D')
            # Doesn't matter what opponent plays now
            P2.history.append(random.choice(['C', 'D']))
            self.assertEqual(P1.grudged, True)

        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.grudged, True)
        P2.history.append('C')

        # Back to being not grudged
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('C')

        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('D')

        # Now grudges again
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.mem_length, 12)
        self.assertEqual(P1.grudged, True)
        P2.history.append('C')

        for turn in range(P1.mem_length-1):
            self.assertEqual(P1.mem_length, 12)
            self.assertEqual(P1.strategy(P2), 'D')
            # Doesn't matter what opponent plays now
            P2.history.append(random.choice(['C', 'D']))
            self.assertEqual(P1.grudged, True)

        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.grudged, True)
        P2.history.append('C')

        # Back to being not grudged
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('C')

    def test_reset_method(self):
        """
        tests the reset method
        """
        P1 = axelrod.InversePunisher()
        P1.history = ['C', 'D', 'D', 'D']
        P1.grudged = True
        P1.grudge_memory = 4
        P1.reset()
        self.assertEqual(P1.history, [])
        self.assertEqual(P1.grudged, False)
        self.assertEqual(P1.grudge_memory, 0)



import random

from axelrod import Player, Game

"""IPD Strategies: http://www.prisoners-dilemma.com/strategies.html"""


class WinStayLoseShift(Player):
    """Win-Stay Lose-Shift, also called Pavlov."""

    name = 'Win-Stay Lose-Shift'

    def __init__(self, initial='C'):
        Player.__init__(self)
        self.response_dict = {
            ('C','C'): 'C',
            ('C','D'): 'D',
            ('D','C'): 'D',
            ('D','D'): 'C',
        }
        self._initial = initial
        self.stochastic = False

    def strategy(self, opponent):
        """Switches if it doesn't get the best payout, traditionally equivalent to a Memory one strategy of [1,0,0,1], but this implementation does not require random draws."""
        if not opponent.history:
            return self._initial
        last_round = (self.history[-1], opponent.history[-1])
        return self.response_dict[last_round]

class MemoryOnePlayer(Player):
    """Uses a four-vector for strategies based on the last round of play, (P(C|CC), P(C|CD), P(C|DC), P(C|DD)), defaults to Win-Stay Lose-Shift. Intended to be used as an abstract base class or to at least be supplied with a initializing four_vector."""

    name = 'Generic Memory One Player'

    def __init__(self, four_vector=[1,0,0,1], initial='C'):
        Player.__init__(self)
        self._four_vector = dict( zip(  [ ('C','C'), ('C','D'), ('D','C'), ('D','D')], map(float, four_vector) ) )
        self._initial = initial
        self.stochastic = False
        for x in set(four_vector):
            if x != 0 and x!= 1:
                self.stochastic = True

    def strategy(self, opponent):
        if not len(opponent.history):
             return self._initial
        # Determine which probability to use
        p = self._four_vector[(self.history[-1], opponent.history[-1])]
        # Draw a random number in [0,1] to decide
        r = random.random()
        if r < p:
            return 'C'
        return 'D'

## Examples of strategies as Memory-One
# tft = (1.,0.,1.,0.)
# gtft = (1.-ep,ep.,1.-ep,ep)
# wsls = (1.,0.,0.,1.)
# alld = (0.,0.,0.,0.)
# allc = (1.,1.,1.,1.)

class GTFT(MemoryOnePlayer):
    """Generous Tit-For-Tat Strategy."""

    name = 'Generous Tit-For-Tat'

    def __init__(self, ep=0.05):
        four_vector = [1-ep, ep, 1-ep, ep]
        super(self.__class__, self).__init__(four_vector)

class StochasticCooperator(MemoryOnePlayer):
    """Stochastic Cooperator, http://www.nature.com/ncomms/2013/130801/ncomms3193/full/ncomms3193.html."""

    name = 'Stochastic Cooperator'

    def __init__(self):
        four_vector = (0.935, 0.229, 0.266, 0.42)
        super(self.__class__, self).__init__(four_vector)

class StochasticWSLS(MemoryOnePlayer):
    """Stochastic WSLS, similar to Generous TFT"""

    name = 'Stochastic WSLS'

    def __init__(self, ep=0.05):
        four_vector = (1.-ep, ep, ep, 1.-ep)
        super(self.__class__, self).__init__(four_vector)

###########################################################
# Have commented out strategies below: see #102 on github #
###########################################################

#class ZDChi(MemoryOnePlayer):
    #"""An Extortionate Zero Determinant Strategy. See the Press and Dyson paper in PNAS for the original formula."""

    #name = 'ZDChi'

    #def __init__(self, chi=2):
        #chi = float(chi)
        #(R, P, T, S) = Game.RPTS()

        #phi_max = float(P-S) / ((P-S) + chi * (T-P))
        #phi = phi_max / 2.

        #p1 = 1. - phi*(chi - 1) * float(R-P) / (P-S)
        #p2 = 1 - phi * (1 + chi * float(T-P) / (P-S))
        #p3 = phi * (chi + float(T-P)/(P-S))
        #p4 = 0

        #four_vector = (p1, p2, p3, p4)
        #super(self.__class__, self).__init__(four_vector)

#def zd_vector2(chi):
    #"""Note that this function assumes the (3,0,5,1) game matrix. It's supposed to enforce s_x - R = 2(S_y - R) but may not. See http://nr.com/whp/StewartPlotkinExtortion2012.pdf"""
    #return (1., (chi - 1.)/(3. * chi + 2.), 1., 2.*(chi - 1.)/(3. * chi + 2.))

#class ZDGTFT2(MemoryOnePlayer):
    #"""A Generous Zero Determinant Strategy."""

    #name = 'ZDGTFT2'

    #def __init__(self):
        #four_vector = zd_vector2(2.)
        #super(self.__class__, self).__init__(four_vector)



from axelrod import Player


class Forgiver(Player):
    """
    A player starts by cooperating however will defect if at any point 
    the opponent has defected more than 10 percent of the time
    """

    name = 'Forgiver'

    def strategy(self, opponent):
        """
        Begins by playing C, then plays D if the opponent has defected more than 10 percent of the time
        """
        try:
            if opponent.history.count('D')>len(opponent.history)/10:
                return 'D'
            return 'C'
        except IndexError:
            return 'C'


class ForgivingTitForTat(Player):
    """
    A player starts by cooperating however will defect if at any point,
    the opponent has defected more than 10 percent of the time,
    and their most recent decision was defect.
    """

    name = 'Forgiving Tit For Tat'

    def strategy(self, opponent):
        """
        Begins by playing C, then plays D if, 
        the opponent has defected more than 10 percent of the time,
        and their most recent decision was defect.
        """
        try:
            if opponent.history.count('D')>len(opponent.history)/10:
                return opponent.history[-1]
            return 'C'
        except IndexError:
            return 'C'



"""A script to run the Axelrod tournament.

The code for strategies is present in `axelrod/strategies`.
"""

from __future__ import division

import argparse
import logging
import axelrod


def run_tournaments(turns, repetitions, exclude_basic, exclude_strategies,
                    exclude_cheating, exclude_all, no_eco, output_directory,
                    logging_option, processes, save_cache):
    manager = axelrod.TournamentManager(
        output_directory=output_directory,
        with_ecological=not no_eco, save_cache=save_cache)

    logHandlers = {
        'console': logging.StreamHandler(),
        'none': logging.NullHandler(),
        'file': logging.FileHandler('./axelrod.log')
    }
    logHandler = logHandlers[logging_option]

    logFormatters = {
        'console': '%(message)s',
        'none': '',
        'file': '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    }
    logFormatter = logging.Formatter(logFormatters[logging_option])

    logHandler.setFormatter(logFormatter)
    logger = logging.getLogger('axelrod')
    logger.setLevel(logging.DEBUG)
    logger.addHandler(logHandler)

    if not exclude_basic:
        players = manager.one_player_per_strategy(axelrod.basic_strategies)
        manager.add_tournament(
            name='basic_strategies', players=players, processes=processes)
    if not exclude_strategies:
        strategies = axelrod.basic_strategies + axelrod.ordinary_strategies
        players = manager.one_player_per_strategy(strategies)
        manager.add_tournament(
            name='strategies', players=players, processes=processes)
    if not exclude_cheating:
        players = manager.one_player_per_strategy(axelrod.cheating_strategies)
        manager.add_tournament(
            name='cheating_strategies', players=players, processes=processes)
    if not exclude_all:
        strategies = (
            axelrod.basic_strategies +
            axelrod.ordinary_strategies +
            axelrod.cheating_strategies)
        players = manager.one_player_per_strategy(strategies)
        manager.add_tournament(
            name='all_strategies', players=players, processes=processes)

    manager.run_tournaments()

if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument('-l', '--logging', type=str, default='console',
                        help='logging (none, console or file)')
    parser.add_argument('-t', '--turns', type=int, default=200,
                        help='turns per pair')
    parser.add_argument('-r', '--repetitions', type=int, default=50,
                        help='round-robin repetitions')
    parser.add_argument('-o', '--output_directory', default='./assets/',
                        help='output directory')
    parser.add_argument('--xb', action='store_true',
                        help='exlude basic strategies plot')
    parser.add_argument('--xs', action='store_true',
                        help='exlude ordinary strategies plot')
    parser.add_argument('--xc', action='store_true',
                        help='exclude cheating strategies plot')
    parser.add_argument('--xa', action='store_true',
                        help='exclude combined strategies plot')
    parser.add_argument('--ne', action='store_true',
                        help='no ecological variant')
    parser.add_argument('-p', '--processes', type=int, default=None,
                        help='Number of parallel processes to spawn. 0 uses cpu count.')
    parser.add_argument('--rc', action='store_true',
                        help='rebuild cache and save to file')
    args = parser.parse_args()

    if args.xb and args.xs and args.xc and args.xa:
        print "You've excluded everything - nothing for me to do"
    else:
        run_tournaments(args.turns, args.repetitions, args.xb, args.xs,
                        args.xc, args.xa, args.ne, args.output_directory,
                        args.logging, args.processes, args.rc)



import unittest
import axelrod


class TestRoundRobin(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.game = axelrod.Game()

    @classmethod
    def payoffs2scores(cls, payoffs):
        return [sum([pp for ipp,pp in enumerate(p) if ipp != ip]) for ip,p in enumerate(payoffs)]

    @classmethod
    def get_test_outcome(cls, outcome, turns=10):

        # Extract the name of players from the outcome tupples,
        # and initiate the players by getting the classes from axelrod.
        names = [out[0] for out in outcome]
        players = [getattr(axelrod, n)() for n in names]

        # Do the actual game and build the expected outcome tuples.
        round_robin = axelrod.RoundRobin(players=players, game=cls.game, turns=turns)
        payoffs = round_robin.play()
        scores = cls.payoffs2scores(payoffs)
        outcome = zip(names, scores)

        # The outcome is expected to be sort by score.
        return sorted(outcome, key = lambda k: k[1])

    def test_init(self):
        p1, p2 = axelrod.Player(), axelrod.Player()
        rr = axelrod.RoundRobin(players=[p1, p2], game=self.game, turns=20)
        self.assertEquals(rr.players, [p1, p2])
        self.assertEquals(rr.nplayers, 2)
        self.assertEquals(rr.game.score(('C', 'C')), (2, 2))

    def test_deterministic_cache(self):
        p1, p2, p3 = axelrod.Cooperator(), axelrod.Defector(), axelrod.Random()
        rr = axelrod.RoundRobin(players=[p1, p2, p3], game=self.game, turns=20)
        self.assertEquals(rr.deterministic_cache, {})
        rr.play()
        self.assertEqual(rr.deterministic_cache[(axelrod.Defector, axelrod.Defector)], (80, 80))
        self.assertEqual(rr.deterministic_cache[(axelrod.Cooperator, axelrod.Cooperator)], (40, 40))
        self.assertEqual(rr.deterministic_cache[(axelrod.Cooperator, axelrod.Defector)], (100, 0))
        self.assertFalse((axelrod.Random, axelrod.Random) in rr.deterministic_cache)

    def test_calculate_score_for_mix(self):
        """Test that scores are calculated correctly."""
        P1 = axelrod.Defector()
        P1.history = ['C', 'C', 'D']
        P2 = axelrod.Defector()
        P2.history = ['C', 'D', 'D']
        round_robin = axelrod.RoundRobin(players=[P1, P2], game=self.game, turns=200)
        self.assertEqual(round_robin.calculate_scores(P1, P2), (11, 6))

    def test_calculate_score_for_all_cooperate(self):
        """Test that scores are calculated correctly."""
        P1 = axelrod.Player()
        P1.history = ['C', 'C', 'C']
        P2 = axelrod.Player()
        P2.history = ['C', 'C', 'C']
        round_robin = axelrod.RoundRobin(players=[P1, P2], game=self.game, turns=200)
        self.assertEqual(round_robin.calculate_scores(P1, P2), (6, 6))

    def test_calculate_score_for_all_defect(self):
        """Test that scores are calculated correctly."""
        P1 = axelrod.Player()
        P1.history = ['D', 'D', 'D']
        P2 = axelrod.Player()
        P2.history = ['D', 'D', 'D']
        round_robin = axelrod.RoundRobin(players=[P1, P2], game=self.game, turns=200)
        self.assertEqual(round_robin.calculate_scores(P1, P2), (12, 12))

    def test_round_robin_defector_v_cooperator(self):
        """Test round robin: the defector viciously punishes the cooperator."""
        outcome = [('Defector', 0), ('Cooperator', 50)]
        self.assertEqual(self.get_test_outcome(outcome), outcome)

    def test_round_robin_defector_v_titfortat(self):
        """Test round robin: the defector does well against tit for tat."""
        outcome = [('Defector', 36), ('TitForTat', 41)]
        self.assertEqual(self.get_test_outcome(outcome), outcome)

    def test_round_robin_cooperator_v_titfortat(self):
        """Test round robin: the cooperator does very well WITH tit for tat."""
        outcome = [('Cooperator', 20), ('TitForTat', 20)]
        self.assertEqual(self.get_test_outcome(outcome), outcome)

    def test_round_robin_cooperator_v_titfortat_v_defector(self):
        """Test round robin: the defector seems to dominate in this small population."""
        outcome = [('Defector', 36), ('TitForTat', 61), ('Cooperator', 70)]
        self.assertEqual(self.get_test_outcome(outcome), outcome)

    def test_round_robin_cooperator_v_titfortat_v_defector_v_grudger(self):
        """Test round robin: tit for tat does a lot better this time around."""
        outcome = [('Defector', 72), ('TitForTat', 81), ('Grudger', 81), ('Cooperator', 90)]
        self.assertEqual(self.get_test_outcome(outcome), outcome)

    def test_round_robin_cooperator_v_titfortat_v_defector_v_grudger_v_go_by_majority(self):
        """Test round robin: Tit for tat now wins."""
        outcome = [('TitForTat', 101), ('Grudger', 101), ('GoByMajority', 101), ('Defector', 108), ('Cooperator', 110)]
        self.assertEqual(self.get_test_outcome(outcome), outcome)



"""Test for the defector strategy."""

import axelrod

from test_player import TestPlayer


class TestDefector(TestPlayer):

    name = "Defector"
    player = axelrod.Defector
    stoachastic = False

    def test_strategy(self):
        """Test that always defects."""

        P1 = axelrod.Defector()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'D')
        P1.history = ['C', 'D', 'C']
        P2.history = ['C', 'C', 'D']
        self.assertEqual(P1.strategy(P2), 'D')


class TestTrickyDefector(TestPlayer):

    name = "Tricky Defector"
    player = axelrod.TrickyDefector
    stochastic = False

    def test_strategy(self):
        """Test if it is trying to trick opponent."""
        P1 = axelrod.TrickyDefector()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'D')
        P1.history = ['D', 'D', 'D', 'D']
        P2.history = ['C', 'D', 'D', 'D']



import random

from axelrod import Player


class Defector(Player):
    """A player who only ever defects."""

    name = 'Defector'

    def strategy(self, opponent):
        return 'D'


class TrickyDefector(Player):
    """A defector that is trying to be tricky."""

    name = "Tricky Defector"

    def strategy(self, opponent):
        """Almost always defects, but will try to trick the opponent into cooperating.

        Defect if opponent has cooperated at least once in the past and has defected
        for the last 3 turns in a row.
        """
        if 'C' in opponent.history and opponent.history[-3:] == ['D']*3:
            return 'C'
        return 'D'


"""Tests for grudger strategies."""

import random

import axelrod

from test_player import TestPlayer


class TestGrudger(TestPlayer):

    name = "Grudger"
    player = axelrod.Grudger
    stochastic = False

    def test_initial_strategy(self):
        """
        Starts by cooperating
        """
        P1 = axelrod.Grudger()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_strategy(self):
        """
        If opponent defects at any point then the player will defect forever
        """
        P1 = axelrod.Grudger()
        P2 = axelrod.Player()
        P1.history = ['C', 'D', 'D', 'D']
        P2.history = ['C', 'C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history = ['C', 'C', 'D', 'D', 'D']
        P2.history = ['C', 'D', 'C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'D')


class TestForgetfulGrudger(TestPlayer):

    name = "Forgetful Grudger"
    player = axelrod.ForgetfulGrudger
    stochastic = False

    def test_strategy(self):

        P1 = axelrod.ForgetfulGrudger()
        P2 = axelrod.Player()

        self.assertEqual(P1.grudged, False)

        # Starts by playing C
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('C')

        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('C')

        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('C')

        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('D')

        self.assertEqual(P2.history, ['C', 'C', 'C', 'D'])
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.grudged, True)

        for turn in range(P1.mem_length-1):
            self.assertEqual(P1.strategy(P2), 'D')
            # Doesn't matter what opponent plays now
            P2.history.append(random.choice(['C', 'D']))
            self.assertEqual(P1.grudged, True)

        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.grudge_memory, 10)
        self.assertEqual(P1.grudged, True)
        P2.history.append('C')

        # Back to being not grudged
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('C')

        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('C')

    def test_reset_method(self):
        """
        tests the reset method
        """
        P1 = axelrod.ForgetfulGrudger()
        P1.history = ['C', 'D', 'D', 'D']
        P1.grudged = True
        P1.grudge_memory = 4
        P1.reset()
        self.assertEqual(P1.history, [])
        self.assertEqual(P1.grudged, False)
        self.assertEqual(P1.grudge_memory, 0)


class TestOppositeGrudger(TestPlayer):

    name = 'Opposite Grudger'
    player = axelrod.OppositeGrudger
    stochastic = False

    def test_initial_strategy(self):
        """
        Starts by cooperating
        """
        P1 = axelrod.OppositeGrudger()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'D')

    def test_strategy(self):
        """
        If opponent cooperates at any point then the player will cooperate forever
        """
        P1 = axelrod.OppositeGrudger()
        P2 = axelrod.Player()
        P1.history = ['C', 'D', 'D', 'D']
        P2.history = ['D', 'D', 'D', 'D']
        self.assertEqual(P1.strategy(P2), 'D')
        P1.history = ['C', 'C', 'D', 'D', 'D']
        P2.history = ['C', 'D', 'C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'C')



"""Test for the forgiver strategies."""

import axelrod

from test_player import TestPlayer


class TestForgiver(TestPlayer):

    def test_initial_strategy(self):
        """Starts by cooperating."""
        P1 = axelrod.Forgiver()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_strategy(self):
        """If opponent has defected more than 10 percent of the time, defect."""
        P1 = axelrod.Forgiver()
        P2 = axelrod.Player()
        P1.history = ['C', 'C', 'C', 'C']
        P2.history = ['C', 'C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history = ['C', 'C', 'C', 'C', 'D']
        P2.history = ['C', 'C', 'C', 'D', 'C']
        self.assertEqual(P1.strategy(P2), 'D')
        P1.history = ['C', 'C', 'C', 'C', 'C', 'C', 'C', 'C', 'C', 'C', 'C']
        P2.history = ['C', 'C', 'C', 'C', 'C', 'C', 'C', 'C', 'C', 'C', 'D']


class TestForgivingTitForTat(TestPlayer):

    def test_initial_strategy(self):
        """Starts by cooperating."""
        P1 = axelrod.ForgivingTitForTat()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_strategy(self):
        """If opponent has defected more than 10 percent of the time, defect."""
        P1 = axelrod.ForgivingTitForTat()
        P2 = axelrod.Player()
        P1.history = ['C', 'C', 'C', 'C']
        P2.history = ['C', 'C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history = ['C', 'C', 'C', 'C', 'D',]
        P2.history = ['C', 'C', 'C', 'D', 'C',]
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history = ['C', 'C', 'C', 'C', 'C', 'C', 'C', 'C', 'C', 'C', 'C']
        P2.history = ['C', 'C', 'C', 'C', 'C', 'C', 'C', 'C', 'C', 'C', 'D']
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history = ['C', 'C', 'C', 'C']
        P2.history = ['C', 'C', 'C', 'D']
        self.assertEqual(P1.strategy(P2), 'D')



"""Test for the geller strategy."""

import axelrod

from test_player import TestPlayer


class TestGeller(TestPlayer):

    name = "Geller"
    player = axelrod.Geller
    stochastic = True

    def test_strategy(self):
        """Should cooperate against cooperaters and defect against defectors."""

        P1 = self.player()
        P2 = axelrod.Cooperator()
        self.assertEqual(P1.strategy(P2), 'C')

        P1 = self.player()
        P2 = axelrod.Defector()
        self.assertEqual(P1.strategy(P2), 'D')

class TestGellerCooperator(TestGeller):

    name = "Geller Cooperator"
    player = axelrod.GellerCooperator
    stochastic = False

    def test_against_self(self):
        P1 = self.player()
        P2 = self.player()
        self.assertEqual(P1.strategy(P2), 'C')

class TestGellerDefector(TestGeller):

    name = "Geller Defector"
    player = axelrod.GellerDefector
    stochastic = False

    def test_against_self(self):
        P1 = self.player()
        P2 = self.player()
        self.assertEqual(P1.strategy(P2), 'D')



from axelrod import Player


class MindController(Player):
    """A player that changes the opponents strategy to cooperate."""

    name = 'Mind Controller'

    def strategy(self, opponent):
        """Alters the opponents strategy method to be a lambda function which always returns C
        This player will then always return D to take advantage of this
        """

        opponent.strategy = lambda opponent: 'C'

        return 'D'

class MindWarper(Player):
    """A player that changes the opponent's strategy but blocks changes to it's own."""

    name = 'Mind Warper'

    def __setattr__(self, name, val):
        if name == 'strategy':
            pass
        else:
            self.__dict__[name] = val

    def strategy(self, opponent):
        opponent.strategy = lambda opponent: 'C'
        return 'D'

class MindBender(MindWarper):
    """A player that changes the opponent's strategy by modifying the internal dictionary."""

    name = 'Mind Bender'

    def strategy(self, opponent):
        opponent.__dict__['strategy'] = lambda opponent: 'C'
        return 'D'



import unittest

import axelrod


class TestResultSet(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.players = ('Player1', 'Player2', 'Player3')
        cls.test_results = [
            [[0, 0], [10, 10], [21, 21]],
            [[10, 8], [0, 0], [16, 20]],
            [[16, 16], [16, 16], [0, 0]],
        ]
        cls.expected_scores = [
            [3.1, 3.1],
            [2.6, 2.8],
            [3.2, 3.2],
        ]
        cls.expected_payoffs = [
            [0.0, 2.0, 4.2],
            [1.8, 0.0, 3.6],
            [3.2, 3.2, 0.0],
        ]
        cls.test_payoffs_list = [
            [[0, 10, 21], [10, 0, 16], [16, 16, 0]],
            [[0, 10, 21], [8, 0, 20], [16, 16, 0]],
        ]
        cls.expected_stddevs = [
            [0.0, 0.0, 0.0],
            [0.20, 0.0, 0.40],
            [0.0, 0.0, 0.0],
        ]
        cls.expected_ranking = [1, 0, 2]
        cls.expected_ranked_names = ['Player2', 'Player1', 'Player3']
        cls.expected_csv = 'Player2,Player1,Player3\n2.6,3.1,3.2\n2.8,3.1,3.2\n'

    def test_init(self):
        rs = axelrod.ResultSet(self.players, 5, 2)
        expected_results = [[[0,0] for j in range(3)] for i in range(3)]
        self.assertEquals(rs.nplayers, 3)
        self.assertEquals(rs.players, self.players)
        self.assertEquals(rs.turns, 5)
        self.assertEquals(rs.repetitions, 2)
        self.assertTrue(rs.results, expected_results)
        self.assertFalse(rs.finalised)

    def test_generate_scores(self):
        rs = axelrod.ResultSet(self.players, 5, 2)
        rs.results = self.test_results
        self.assertEquals(rs.generate_scores(), self.expected_scores)

    def test_generate_ranking(self):
        rs = axelrod.ResultSet(self.players, 5, 2)
        rs.results = self.test_results
        scores = rs.generate_scores()
        self.assertEquals(rs.generate_ranking(scores), self.expected_ranking)

    def test_generate_ranked_names(self):
        rs = axelrod.ResultSet(self.players, 5, 2)
        rs.results = self.test_results
        scores = rs.generate_scores()
        rankings = rs.generate_ranking(scores)
        self.assertEquals(rs.generate_ranked_names(rankings), self.expected_ranked_names)

    def test_generate_payoff_matrix(self):
        rs = axelrod.ResultSet(self.players, 5, 2)
        rs.results = self.test_results
        payoffs, stddevs = rs.generate_payoff_matrix()
        stddevs = [[round(x, 1) for x in row] for row in stddevs]
        self.assertEquals(payoffs, self.expected_payoffs)
        self.assertEquals(stddevs, self.expected_stddevs)

    def test_finalise(self):
        rs = axelrod.ResultSet(self.players, 5, 2)
        rs.finalise(self.test_payoffs_list)
        self.assertEquals(rs.scores, self.expected_scores)
        self.assertEquals(rs.ranking, self.expected_ranking)
        self.assertEquals(rs.ranked_names, self.expected_ranked_names)
        self.assertTrue(rs.finalised)
        self.assertRaises(AttributeError, rs.finalise, self.test_payoffs_list)

    def test_csv(self):
        rs = axelrod.ResultSet(self.players, 5, 2)
        self.assertRaises(AttributeError, rs.csv)
        rs.finalise(self.test_payoffs_list)
        rs.results = self.test_results
        self.assertEquals(rs.csv(), self.expected_csv)



"""Test for the mindreader strategy."""

import axelrod

from test_player import TestPlayer


class TestMindReader(TestPlayer):

    name = "Mind Reader"
    player = axelrod.MindReader
    stochastic = False

    def test_strategy(self):
        """
        Will defect against nice strategies
        """
        P1 = axelrod.MindReader()
        P2 = axelrod.Cooperator()
        self.assertEqual(P1.strategy(P2), 'D')
 
    def test_vs_defect(self):
        """
        Will defect against pure defecting strategies
        """
        P1 = axelrod.MindReader()
        P2 = axelrod.Defector()
        self.assertEqual(P1.strategy(P2), 'D')

    def test_vs_grudger(self):
        """
        Will keep nasty strategies happy if it can
        """
        P1 = axelrod.MindReader()
        P2 = axelrod.Grudger()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_vs_tit_for_tat(self):
        """
        Will keep nasty strategies happy if it can
        """
        P1 = axelrod.MindReader()
        P2 = axelrod.Grudger()
        self.assertEqual(P1.strategy(P2), 'C')
 
    def test_simulate_matches(self):
        """
        Simulates a number of matches
        """
        P1 = axelrod.MindReader()
        P2 = axelrod.Grudger()
        P1.simulate_match(P2, 'C', 4)
        self.assertEqual(P2.history, ['C', 'C', 'C', 'C'])
 
    def test_history_is_same(self):
        """
        Checks that the history is not altered by the player
        """
        P1 = axelrod.MindReader()
        P2 = axelrod.Grudger()
        P1.history = ['C', 'C']
        P2.history = ['C', 'D']
        P1.strategy(P2)
        self.assertEqual(P1.history, ['C', 'C'])
        self.assertEqual(P2.history, ['C', 'D'])

    def test_vs_geller(self):
        """Ensures that a recursion error does not occour """
        P1 = axelrod.MindReader()
        P2 = axelrod.Geller()
        P1.strategy(P2)
        P2.strategy(P1)
        
    def tests_protected_mind_reader(self):
        """Ensures that no other player can alter its strategy """

        P1 = axelrod.ProtectedMindReader()
        P2 = axelrod.MindController()
        P3 = axelrod.Cooperator()
        P2.strategy(P1)
        self.assertEqual(P1.strategy(P3), 'D')

    def test_init(self):
        """Tests for init method """
        P1 = axelrod.MindReader()
        self.assertEqual(P1.history, [])

    def test_reset(self):
        """Tests to see if the class is reset correctly """
        P1 = axelrod.MindReader()
        P1.history = ['C', 'D', 'D', 'D']
        P1.reset()
        self.assertEqual(P1.history, [])



from axelrod import Player


class Retaliate(Player):
    """
    A player starts by cooperating but will retaliate once the opponent
    has won more than 10 percent times the number of defections the player has.
    """
    retaliation_threshold = 0.1

    def __init__(self):
        """
        Uses the basic init from the Player class, but also set the name to
        include the retaliation setting.
        """
        Player.__init__(self)
        self.name = (
            'Retaliate (' +
            str(self.retaliation_threshold) + ')')

    def strategy(self, opponent):
        """
        If the opponent has played D to my C more often than x% of the time
        that I've done the same to him, play D. Otherwise, play C.
        """
        history = zip(self.history, opponent.history)
        if history.count(('C', 'D')) > (
           history.count(('D', 'C')) * self.retaliation_threshold):
            return 'D'
        return 'C'


class Retaliate2(Retaliate):
    retaliation_threshold = 0.08


class Retaliate3(Retaliate):
    retaliation_threshold = 0.05


class LimitedRetaliate(Player):
    """
    A player that co-operates unless the opponent defects and wins.
    It will then retaliate by defecting. It stops when either, it has beaten
    the opponent 10 times more often that it has lost or it reaches the
    retaliation limit (20 defections).
    """
    retaliation_limit = 20
    retaliation_threshold = 0.1
    retaliating = False
    retaliation_count = 0

    def __init__(self):
        """
        Uses the basic init from the Player class, but also set the name to
        include the retaliation setting.
        """
        Player.__init__(self)
        self.name = (
            'Limited Retaliate (' +
            str(self.retaliation_threshold) +
            '/' + str(self.retaliation_limit) + ')')

    def strategy(self, opponent):
        """
        If the opponent has played D to my C more often than x% of the time
        that I've done the same to him, retaliate by playing D but stop doing
        so once I've hit the retaliation limit.
        """
        history = zip(self.history, opponent.history)
        if history.count(('C', 'D')) > (
           history.count(('D', 'C')) * self.retaliation_threshold):
            self.retaliating = True
        else:
            self.retaliating = False
            self.retaliation_count = 0

        if self.retaliating:
            if self.retaliation_count < self.retaliation_limit:
                self.retaliation_count += 1
                return 'D'
            else:
                self.retaliation_count = 0
                self.retaliating = False

        return 'C'

    def reset(self):
        Player.reset(self)
        self.retaliating = False
        self.retaliation_count = 0


class LimitedRetaliate2(LimitedRetaliate):
    retaliation_limit = 15
    retaliation_threshold = 0.08


class LimitedRetaliate3(LimitedRetaliate):
    retaliation_limit = 20
    retaliation_threshold = 0.05



"""Tests for the hunter strategy."""

import random

import axelrod

from test_player import TestPlayer


class TestDefectorHunter(TestPlayer):

    name = "Defector Hunter"
    player = axelrod.DefectorHunter
    stochastic = False

    def test_strategy(self):

        P1 = axelrod.DefectorHunter()
        P2 = axelrod.Player()
        for i in range(4):
            self.assertEqual(P1.strategy(P2), 'C')
            P1.history.append('C')
            P2.history.append('D')
        self.assertEqual(P1.strategy(P2), 'D')

class TestCooperatorHunter(TestPlayer):

    name = "Cooperator Hunter"
    player = axelrod.CooperatorHunter
    stochastic = False

    def test_strategy(self):

        P1 = axelrod.CooperatorHunter()
        P2 = axelrod.Player()
        for i in range(4):
            self.assertEqual(P1.strategy(P2), 'C')
            P1.history.append('C')
            P2.history.append('C')
        self.assertEqual(P1.strategy(P2), 'D')

class TestAlternatorHunter(TestPlayer):

    name = "Alternator Hunter"
    player = axelrod.AlternatorHunter
    stochastic = False

    def test_strategy(self):

        P1 = axelrod.AlternatorHunter()
        P2 = axelrod.Player()
        for i in range(4):
            self.assertEqual(P1.strategy(P2), 'C')
            P1.history.append('C')
            P2.history.append('C'*(i%2==0) or 'D')
        self.assertEqual(P1.strategy(P2), 'D')

class TestMathConstantHunter(TestPlayer):

    name = "Math Constant Hunter"
    player = axelrod.MathConstantHunter
    stochastic = False

    def test_strategy(self):

        P1 = axelrod.MathConstantHunter()
        P2 = axelrod.Player()
        P1.history = ['C'] * 8
        P2.history = ['C', 'C', 'C', 'D', 'C', 'C', 'C', 'D']
        self.assertEqual(P1.strategy(P2), 'D')

class TestRandomHunter(TestPlayer):

    name = "Random Hunter"
    player = axelrod.RandomHunter
    stochastic = False

    def test_strategy(self):

        P1 = axelrod.RandomHunter()
        P2 = axelrod.Player()
        P1.history = ['C'] * 8
        P2.history = [random.choice(['C', 'D']) for i in range(8)]
        for i in range(10):
            self.assertEqual(P1.strategy(P2), 'D')
            P1.history.append('D')
            P2.history.append(random.choice(['C', 'D']))

class TestMetaHunter(TestPlayer):

    name = "Meta Hunter"
    player = axelrod.MetaHunter
    stochastic = False

    def test_strategy(self):

        P1 = axelrod.MetaHunter()
        P2 = axelrod.Player()

        self.assertEqual(P1.strategy(P2), 'C')

        # We are not using the Cooperator Hunter here, so this should lead to cooperation.
        P1.history = ['C'] * 4
        P2.history = ['C'] * 4
        self.assertEqual(P1.strategy(P2), 'C')

        # All these others, however, should trigger a defection for the hunter.
        histories = [
            ['D'] * 4,
            ['C', 'D'] * 2,
            ['C', 'C', 'C', 'D', 'C', 'C', 'C', 'D'],
            [random.choice(['C', 'D']) for i in range(8)],
        ]
        for h in histories:
            P1.history = ['C'] * len(h)
            P2.history = h
            self.assertEqual(P1.strategy(P2), 'D')



"""Tests for the hunter strategy."""

import axelrod

from test_player import TestPlayer


class TestMetaMajority(TestPlayer):

    name = "Meta Majority"
    player = axelrod.MetaMajority
    stochastic = True

    def test_strategy(self):

        P1 = axelrod.MetaMajority()
        P2 = axelrod.Player()

        # With more cooperators on the team than defectors, we should cooperate.
        P1.team = [axelrod.Cooperator(), axelrod.Cooperator(), axelrod.Defector()]
        self.assertEqual(P1.strategy(P2), "C")

        # With more defectors, we should defect.
        P1.team = [axelrod.Cooperator(), axelrod.Defector(), axelrod.Defector()]
        self.assertEqual(P1.strategy(P2), "D")


class TestMetaMinority(TestPlayer):

    name = "Meta Minority"
    player = axelrod.MetaMinority
    stochastic = True

    def test_strategy(self):

        P1 = axelrod.MetaMinority()
        P2 = axelrod.Player()

        # With more cooperators on the team, we should defect.
        P1.team = [axelrod.Cooperator(), axelrod.Cooperator(), axelrod.Defector()]
        self.assertEqual(P1.strategy(P2), "D")

        # With defectors in the majority, we will cooperate here.
        P1.team = [axelrod.Cooperator(), axelrod.Defector(), axelrod.Defector()]
        self.assertEqual(P1.strategy(P2), "C")


class TestMetaWinner(TestPlayer):

    name = "Meta Winner"
    player = axelrod.MetaWinner
    stochastic = True

    def test_strategy(self):

        P1 = axelrod.MetaWinner(team = [axelrod.Cooperator, axelrod.Defector])
        P2 = axelrod.Player()

        # This meta player will simply choose the strategy with the highest current score.
        P1.team[0].score = 0
        P1.team[1].score = 1
        self.assertEqual(P1.strategy(P2), 'C')
        P1.team[0].score = 1
        P1.team[1].score = 0
        self.assertEqual(P1.strategy(P2), 'D')

        # If there is a tie, choose to cooperate if possible.
        P1.team[0].score = 1
        P1.team[1].score = 1
        self.assertEqual(P1.strategy(P2), 'C')


import random
from axelrod import Player

class OnceBitten(Player):
    """
    Co-operates once when the opponent defects, but if they defect twice in a row defaults to forgetful grudger for 10 turns defecting
    """

    name = 'Once Bitten'

    def __init__(self):
        """
        Initialised the player
        """
        super(OnceBitten, self).__init__()
        self.history = []
        self.score = 0
        self.mem_length = 10
        self.grudged = False
        self.grudge_memory = 0
        self.stochastic = False

    def strategy(self, opponent):
        """
        Begins by playing C, then plays D for mem_length rounds if the opponent ever plays D twice in a row
        """
        if self.grudge_memory >= self.mem_length:
            self.grudge_memory = 0
            self.grudged = False

        if len(opponent.history) < 2:
            return 'C'

        if self.grudged:
            self.grudge_memory += 1
            return 'D'
        elif not ('C' in opponent.history[-2:]):
            self.grudged = True
            return 'D'
        return 'C'

    def reset(self):
        """
        Resets scores and history
        """
        self.history = []
        self.grudged = False
        self.grudge_memory = 0

class FoolMeOnce(Player):
    """
    Forgives one D then retaliates forever on a second D.
    """

    name = 'Fool Me Once'

    def __init__(self):
        """
        Initialised the player
        """
        super(FoolMeOnce, self).__init__()
        self.D_count = 0
        self._initial = 'C'
        self.stochastic = False

    def strategy(self, opponent):
        if not opponent.history:
            return self._initial
        if opponent.history[-1] == 'D':
            self.D_count += 1
        if self.D_count > 1:
            return 'D'
        return 'C'

    def reset(self):
        self.D_count = 0
        self.history = []

class ForgetfulFoolMeOnce(Player):
    """
    Forgives one D then retaliates forever on a second D. Sometimes randomly forgets the defection count.
    """

    name = 'Forgetful Fool Me Once'

    def __init__(self, forget_probability=0.1):
        super(ForgetfulFoolMeOnce, self).__init__()
        self.D_count = 0
        self._initial = 'C'
        self.forget_probability = forget_probability
        self.stochastic = True

    def strategy(self, opponent):
        r = random.random()
        if not opponent.history:
            return self._initial
        if opponent.history[-1] == 'D':
            self.D_count += 1
        if r < self.forget_probability:
            self.D_count = 0
        if self.D_count > 1:
            return 'D'
        return 'C'

    def reset(self):
        self.D_count = 0
        self.history = []



"""Test for the average_copier strategy."""

import random

import axelrod

from test_player import TestPlayer


class TestAverageCopier(TestPlayer):

    name = "Average Copier"
    player = axelrod.AverageCopier
    stochastic = True

    def test_strategy(self):
        """Test that the first strategy is picked randomly."""
        random.seed(1)
        P1 = axelrod.AverageCopier()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')

    def test_when_oppenent_all_Cs(self):
        """
        Tests that if opponent has played all C then player chooses C
        """
        random.seed(5)
        P1 = axelrod.AverageCopier()
        P2 = axelrod.Player()
        P2.history = ['C', 'C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')

    def test_when_opponent_all_Ds(self):
        """
        Tests that if opponent has played all D then player chooses D
        """
        random.seed(5)
        P1 = axelrod.AverageCopier()
        P2 = axelrod.Player()
        P2.history = ['D', 'D', 'D', 'D']
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')


class TestNiceAverageCopier(TestPlayer):

    name = "Nice Average Copier"
    player = axelrod.NiceAverageCopier
    stochastic = True

    def test_strategy(self):
        """Test that the first strategy is cooperation."""
        P1 = axelrod.NiceAverageCopier()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_when_oppenent_all_Cs(self):
        """
        Tests that if opponent has played all C then player chooses C
        """
        random.seed(5)
        P1 = axelrod.NiceAverageCopier()
        P2 = axelrod.Player()
        P2.history = ['C', 'C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.strategy(P2), 'C')

    def test_when_opponent_all_Ds(self):
        """
        Tests that if opponent has played all D then player chooses D
        """
        random.seed(5)
        P1 = axelrod.NiceAverageCopier()
        P2 = axelrod.Player()
        P2.history = ['D', 'D', 'D', 'D']
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.strategy(P2), 'D')


from axelrod import Player


class Punisher(Player):
    """
    A player starts by cooperating however will defect if at any point the opponent has defected,
    but forgets after meme_length matches, with 1<=mem_length<=20 proportional to the amount
    of time the opponent has played 'D', punishing that player for playing 'D' too often
    """

    name = 'Punisher'

    def __init__(self):
        """
        Initialised the player
        """
        super(Punisher, self).__init__()
        self.history = []
        self.score = 0
        self.mem_length = 1
        self.grudged = False
        self.grudge_memory = 1

    def strategy(self, opponent):
        """
        Begins by playing C, then plays D for an amount of rounds proportional to the opponents historical '%' of playing 'D' if the opponent ever plays D
        """

        if self.grudge_memory >= self.mem_length:
            self.grudge_memory = 0
            self.grudged = False

        if self.grudged:
            self.grudge_memory += 1
            return 'D'
        elif 'D' in opponent.history[-1:]:
            self.mem_length = (sum([i == 'D' for i in opponent.history])*20)/len(opponent.history)
            self.grudged = True
            return 'D'
        return 'C'

    def reset(self):
        """
        Resets scores and history
        """
        self.history = []
        self.grudged = False
        self.grudge_memory = 0
        self.mem_length = 1


class InversePunisher(Player):
    """
    A player starts by cooperating however will defect if at any point the opponent has defected,
    but forgets after meme_length matches, with 1<=mem_length<=20 proportional to the amount
    of time the opponent has played 'C'. The inverse of Punisher
    """

    name = 'Inverse Punisher'

    def __init__(self):
        """
        Initialised the player
        """
        super(InversePunisher, self).__init__()
        self.history = []
        self.score = 0
        self.mem_length = 1
        self.grudged = False
        self.grudge_memory = 1

    def strategy(self, opponent):
        """
        Begins by playing C, then plays D for an amount of rounds proportional to the opponents historical '%' of playing 'C' if the opponent ever plays D
        """

        if self.grudge_memory >= self.mem_length:
            self.grudge_memory = 0
            self.grudged = False

        if self.grudged:
            self.grudge_memory += 1
            return 'D'
        elif 'D' in opponent.history[-1:]:
            self.mem_length = ((sum([i == 'C' for i in opponent.history]))*20)/len(opponent.history)
            if self.mem_length == 0:
                self.mem_length += 1
            self.grudged = True
            return 'D'
        return 'C'

    def reset(self):
        """
        Resets scores and history
        """
        self.history = []
        self.grudged = False
        self.grudge_memory = 0
        self.mem_length = 1



from axelrod import Player
import random

class Random(Player):
    """A player who randomly chooses between cooperating and defecting."""

    name = 'Random'

    def strategy(self, opponent):
        return random.choice(['C','D'])



from alternator import *
from appeaser import *
from averagecopier import *
from cooperator import *
from darwin import *
from defector import *
from forgiver import *
from geller import *
from gobymajority import *
from grudger import *
from grumpy import *
from hunter import *
from inverse import *
from mathematicalconstants import *
from memoryone import *
from meta import *
from mindcontrol import *
from mindreader import *
from oncebitten import *
from punisher import *
from qlearner import *
from rand import *
from retaliate import *
from titfortat import *

basic_strategies = [
    Alternator,
    Cooperator,
    Defector,
    Random,
    TitForTat,
]

ordinary_strategies = [
    AlternatorHunter,
    Appeaser,
    ArrogantQLearner,
    AverageCopier,
    CautiousQLearner,
    CooperatorHunter,
    DefectorHunter,
    FoolMeOnce,
    ForgetfulFoolMeOnce,
    ForgetfulGrudger,
    Forgiver,
    ForgivingTitForTat,
    GTFT,
    GoByMajority,
    GoByMajority10,
    GoByMajority20,
    GoByMajority40,
    GoByMajority5,
    Golden,
    Grudger,
    Grumpy,
    HesitantQLearner,
    Inverse,
    InversePunisher,
    LimitedRetaliate,
    LimitedRetaliate2,
    LimitedRetaliate3,
    MathConstantHunter,
    MetaHunter,
    MetaMajority,
    MetaMinority,
    MetaWinner,
    NiceAverageCopier,
    OnceBitten,
    OppositeGrudger,
    Pi,
    Punisher,
    RandomHunter,
    Retaliate,
    Retaliate2,
    Retaliate3,
    RiskyQLearner,
    SneakyTitForTat,
    StochasticWSLS,
    SuspiciousTitForTat,
    TitFor2Tats,
    TrickyCooperator,
    TrickyDefector,
    TwoTitsForTat,
    WinStayLoseShift,
    e,
]

# These are strategies that do not follow the rules of Axelrods tournament
cheating_strategies = [
    Darwin,
    Geller,
    GellerCooperator,
    GellerDefector,
    MindBender,
    MindController,
    MindReader,
    MindWarper,
    ProtectedMindReader,
]



from axelrod import Player

class Alternator(Player):
    """A player who alternates between cooperating and defecting."""

    name = 'Alternator'

    def strategy(self, opponent):
        if len(self.history) == 0:
            return 'C'
        if self.history[-1] == 'C':
            return 'D'
        return 'C'



"""
    Tests for the Darwin PD strategy.
"""

import axelrod
from test_player import TestPlayer

class TestDarwin(TestPlayer):

    name = "Darwin"
    player = axelrod.Darwin

    def test_strategy(self):
        p1 = self.player()
        p2 = axelrod.Player()
        self.assertEqual(p1.strategy(p2), 'C') # Always cooperate first.


    def test_play(self):
        """valid_callers must contain at least one entry..."""
        self.assertTrue(len(self.player.valid_callers)>0)
        """...and should allow round_robin.play to call"""
        self.assertTrue("play" in self.player.valid_callers)
        self.play()


    def play(self):
        """We need this to circumvent the agent's anti-inspection measure"""
        p1 = self.player()
        p2 = axelrod.Player()
        p1.reset()
        p1.strategy(p2)
        # Genome contains only valid responses.
        self.assertEqual(p1.genome.count('C') + p1.genome.count('D'), len(p1.genome))


    def test_reset(self):
        """Is instance correctly reset between rounds"""
        p1 = self.player()
        p1.reset()
        self.assertEqual(p1.history, [])
        self.assertEqual(p1.genome[0], 'C')


    def test_unique_genome(self):
        """Ensure genome remains unique class property"""
        p1 = self.player()
        p2 = self.player()
        self.assertTrue(p1.genome is p2.genome)



import multiprocessing
from game import *
from result_set import *
from round_robin import *
import logging

class Tournament(object):
    game = Game()

    def __init__(self, players, name='axelrod', game=None, turns=200,
                 repetitions=10, processes=None, prebuilt_cache=False):
        self.name = name
        self.players = players
        self.nplayers = len(self.players)

        if game is not None:
            self.game = game
        self.turns = turns
        self.repetitions = repetitions
        self.processes = processes
        self.prebuilt_cache=prebuilt_cache

        self.logger = logging.getLogger(__name__)

        self.result_set = ResultSet(
            players=players,
            turns=turns,
            repetitions=repetitions)

        self.deterministic_cache = {}

    def play(self):
        payoffs_list = []

        if self.processes is None:
            self.run_serial_repetitions(payoffs_list)
        else:
            if len(self.deterministic_cache) == 0 or not self.prebuilt_cache:
                self.logger.debug('Playing first round robin to build cache')
                payoffs = self.play_round_robin()
                payoffs_list.append(payoffs)
                self.repetitions -= 1
            self.run_parallel_repetitions(payoffs_list)

        self.result_set.finalise(payoffs_list)
        return self.result_set

    def run_serial_repetitions(self, payoffs_list):
        self.logger.debug('Playing %d round robins' % self.repetitions)
        for repetition in range(self.repetitions):
            payoffs = self.play_round_robin()
            payoffs_list.append(payoffs)
        return True

    def run_parallel_repetitions(self, payoffs_list):
        # At first sight, it might seem simpler to use the multiprocessing Pool
        # Class rather than Processes and Queues. However, Pool can only accept
        # target functions which can be pickled and instance methods cannot.
        work_queue = multiprocessing.Queue()
        done_queue = multiprocessing.Queue()

        if self.processes < 2 or self.processes > multiprocessing.cpu_count():
            workers = multiprocessing.cpu_count()
        else:
            workers = self.processes

        for repetition in range(self.repetitions):
            work_queue.put(repetition)

        self.logger.debug(
            'Playing %d round robins with %d parallel processes' % (self.repetitions, workers))
        self.start_workers(workers, work_queue, done_queue)
        self.process_done_queue(workers, done_queue, payoffs_list)

        return True

    def start_workers(self, workers, work_queue, done_queue):
        for worker in range(workers):
            process = multiprocessing.Process(
                target=self.worker, args=(work_queue, done_queue))
            work_queue.put('STOP')
            process.start()
        return True

    def process_done_queue(self, workers, done_queue, payoffs_list):
        stops = 0
        while stops < workers:
            payoffs = done_queue.get()
            if payoffs == 'STOP':
                stops += 1
            else:
                payoffs_list.append(payoffs)
        return True

    def worker(self, work_queue, done_queue):
        for repetition in iter(work_queue.get, 'STOP'):
            payoffs = self.play_round_robin(cache_mutable=False)
            done_queue.put(payoffs)
        done_queue.put('STOP')
        return True

    def play_round_robin(self, cache_mutable=True):
        round_robin = RoundRobin(
            players=self.players,
            game=self.game,
            turns=self.turns,
            deterministic_cache=self.deterministic_cache,
            cache_mutable=cache_mutable)
        payoffs = round_robin.play()
        return payoffs



"""Test for the retaliate strategy."""

import axelrod

from test_player import TestPlayer


class TestRetaliate(TestPlayer):

    name = "Retaliate (0.1)"
    player = axelrod.Retaliate

    def test_strategy(self):
        """Starts by cooperating.        """
        P1 = axelrod.Retaliate()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_effect_of_strategy(self):
        """If opponent has defected more than 10 percent of the time, defect."""
        P1 = axelrod.Retaliate()
        P2 = axelrod.Player()
        P1.history = ['C', 'C', 'C', 'C']
        P2.history = ['C', 'C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history = ['C', 'C', 'C', 'C', 'D']
        P2.history = ['C', 'C', 'C', 'D', 'C']
        self.assertEqual(P1.strategy(P2), 'D')
        P1.history = ['C', 'C', 'C', 'C', 'C', 'C']
        P2.history = ['C', 'C', 'C', 'C', 'C', 'D']
        self.assertEqual(P1.strategy(P2), 'D')


class TestLimitedRetaliate(TestPlayer):

    name = 'Limited Retaliate (0.1/20)'
    player = axelrod.LimitedRetaliate

    def test_strategy(self):
        """Starts by cooperating.        """
        P1 = axelrod.LimitedRetaliate()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_effect_of_strategy(self):
        """If opponent has never defected, co-operate"""
        P1 = axelrod.LimitedRetaliate()
        P2 = axelrod.Player()
        P1.history = ['C', 'C', 'C', 'C']
        P2.history = ['C', 'C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertFalse(P1.retaliating)

        """If opponent has previously defected and won, defect and be retaliating"""
        P1.history = ['C', 'C', 'C', 'C', 'D']
        P2.history = ['C', 'C', 'C', 'D', 'C']
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertTrue(P1.retaliating)

        """If opponent has just defected and won, defect and be retaliating"""
        P1.history = ['C', 'C', 'C', 'C', 'C', 'C']
        P2.history = ['C', 'C', 'C', 'C', 'C', 'D']
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertTrue(P1.retaliating)

        """If I've hit the limit for retaliation attempts, co-operate"""
        P1.history = ['C', 'C', 'C', 'C', 'D']
        P2.history = ['C', 'C', 'C', 'D', 'C']
        P1.retaliation_count = 20
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertFalse(P1.retaliating)

    def test_reset(self):
        P1 = axelrod.LimitedRetaliate()
        P1.history = ['C', 'C', 'C', 'C', 'D']
        P1.retaliating = True
        P1.retaliation_count = 4
        P1.reset()
        self.assertEqual(P1.history, [])
        self.assertFalse(P1.retaliating)
        self.assertEqual(P1.retaliation_count, 0)



import random

from axelrod import Player

from meta import MetaPlayer


class DefectorHunter(Player):
    """A player who hunts for defectors."""

    name = 'Defector Hunter'

    def strategy(self, opponent):
        if len(self.history) >= 4 and all([h == 'D' for h in opponent.history]):
            return 'D'
        return 'C'


class CooperatorHunter(Player):
    """A player who hunts for cooperators."""

    name = 'Cooperator Hunter'

    def strategy(self, opponent):
        if len(self.history) >= 4 and all([h == 'C' for h in opponent.history]):
            return 'D'

        return 'C'


class AlternatorHunter(Player):
    """A player who hunts for alternators."""

    name = 'Alternator Hunter'

    def strategy(self, opponent):
        oh = opponent.history
        if len(self.history) >= 4 and all([oh[i] != oh[i+1] for i in range(len(oh)-1)]):
            return 'D'
        return 'C'


class MathConstantHunter(Player):
    """A player who hunts for mathemtical constant players."""

    name = "Math Constant Hunter"

    def strategy(self, opponent):

        n = len(self.history)
        if n >= 8 and 'C' in opponent.history and 'D' in opponent.history:

            # The variance of the uniform distribution is about 1/4, so we can expect
            # that the difference in proportion of defects/cooperations in two halves
            # will quickly vary by less than that.
            delta = abs(opponent.history[:n/2].count('D') - opponent.history[n/2:].count('D'))
            if delta < 0.25 * n:
                return 'D'

        return 'C'


class RandomHunter(Player):
    """A player who hunts for random players."""

    name = "Random Hunter"

    # We need to make sure this is not marked as stochastic.
    def __init__(self):
        Player.__init__(self)
        self.stochastic = False

    def strategy(self, opponent):

        n = len(self.history)
        if n >= 8 and 'C' in opponent.history and 'D' in opponent.history:

            # In terms of rewards, random players will produce unjustified defections,
            # more or less as often as expected by pure chance.
            justified = [i for i in range(1, n) if opponent.history[i] == 'D' and self.history[i-1] == 'D']
            expected = 1.0 * opponent.history.count('D') * self.history.count('D') / n
            if len(justified) < expected + 3:
                return 'D'

        return 'C'


class MetaHunter(MetaPlayer):
    """A player who uses a selection of hunters sequentially."""

    name = "Meta Hunter"

    def __init__(self):

        # We need to make sure this is not marked as stochastic.
        self.stochastic = False

        # Notice that we don't include the cooperator hunter, because it leads to excessive
        # defection and therefore bad performance against unforgiving strategies. We will stick
        # to hunters that use defections as cues. However, a really tangible benefit comes from
        # combining Random Hunter and Math Constant Hunter, since together they catch strategies
        # that are lightly randomized but still quite constant (the tricky/suspecious ones).
        self.team = [DefectorHunter, AlternatorHunter, MathConstantHunter, RandomHunter]

        MetaPlayer.__init__(self)

    def meta_strategy(self, results):

        # If any of the hunters smells prey, then defect!
        if 'D' in results:
            return 'D'
        return 'C'


class RoundRobin(object):
    """A class to define play a round robin game of players"""

    def __init__(self, players, game, turns, deterministic_cache={},
                 cache_mutable=True):
        """Initialise the players, game and deterministic cache"""
        self.players = players
        self.nplayers = len(players)
        self.game = game
        self.turns = turns
        self.deterministic_cache = deterministic_cache
        self.cache_mutable = cache_mutable

    def calculate_scores(self, p1, p2):
        """Calculates the score for two players based their history"""
        s1, s2 = 0, 0
        for pair in zip(p1.history, p2.history):
            score = self.game.score(pair)
            s1 += score[0]
            s2 += score[1]
        return s1, s2

    def play(self):
        """Plays a round robin where each match lasts turns.

        We cache scores for pairs of deterministic strategies, since the outcome will
        always be the same.

        Notice also that we need to handle self-interactions with some special exceptions
        due to the way gameplay is coded within Player.

        Returns the total payoff matrix.
        """

        payoffs = [[0 for j in range(self.nplayers)] for i in range(self.nplayers)]

        for ip1 in range(self.nplayers):

            p1 = self.players[ip1]
            cl1 = p1.__class__

            for ip2 in range(ip1, self.nplayers):

                # For self-interactions we need to create an additional object. otherwise
                # the play method in Player will write twice to the same history, effectively
                # doubling the score and causing historic schizophrenia.
                if ip1 == ip2:
                    p2 = cl1()
                    cl2 = cl1
                else:
                    p2 = self.players[ip2]
                    cl2 = p2.__class__

                # There are many possible keys to cache by, but perhaps the
                # most versatile is a tuple with the classes of both players.
                key = (cl1, cl2)
                if (p1.stochastic or p2.stochastic or key not in self.deterministic_cache):
                    turn = 0
                    p1.reset()
                    p2.reset()
                    while turn < self.turns:
                        turn += 1
                        p1.play(p2)
                    scores = self.calculate_scores(p1, p2)
                    if self.cache_mutable and not (p1.stochastic or p2.stochastic):
                        self.deterministic_cache[key] = scores
                else:
                    scores = self.deterministic_cache[key]

                # For self-interactions we can take the average of the two sides, which
                # should improve the averaging a bit.
                if ip1 == ip2:
                    payoffs[ip1][ip2] = 0.5 * (scores[0] + scores[1])
                else:
                    payoffs[ip1][ip2] = scores[0]
                    payoffs[ip2][ip1] = scores[1]

        return payoffs



"""Test for the cooperator strategy."""

import axelrod

from test_player import TestPlayer


class TestCooperator(TestPlayer):

    name = "Cooperator"
    player = axelrod.Cooperator
    stochastic = False

    def test_strategy(self):
        """Test that always cooperates."""
        P1 = axelrod.Cooperator()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history = ['C', 'D', 'C']
        P2.history = ['C', 'C', 'D']
        self.assertEqual(P1.strategy(P2), 'C')


class TestTrickyCooperator(TestPlayer):

    name = "Tricky Cooperator"
    player = axelrod.TrickyCooperator
    stochastic = False

    def test_strategy(self):
        """Test if it tries to trick opponent"""
        P1 = axelrod.TrickyCooperator()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history = ['C', 'C', 'C']
        P2.history = ['C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'D')
        P1.history.extend(['D', 'D'])
        P2.history.extend(['C', 'D'])
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history.extend(['C']*11)
        P2.history.extend(['D'] + ['C']*10)
        self.assertEqual(P1.strategy(P2), 'D')



"""Test for the golden and other mathematical strategies."""

import axelrod

from test_player import TestPlayer


class TestGolden(TestPlayer):

    name = '$\phi$'
    player = axelrod.Golden

    def test_strategy(self):
        """test initial strategy co-operates"""
        P1 = axelrod.Golden()
        P2 = axelrod.Player()
        P2.history = []
        self.assertEqual(P1.strategy(P2), 'C')

    def test_when_no_defection(self):
        """tests that if the opposing player does not defect initially then strategy defects"""
        P1 = axelrod.Golden()
        P2 = axelrod.Player()
        P1.history = ['C']
        P2.history = ['C']
        self.assertEqual(P1.strategy(P2), 'D')

    def test_when_greater_than_golden_ratio(self):
        """tests that if the ratio of Cs to Ds is greater than the golden ratio then strategy defects"""
        P1 = axelrod.Golden()
        P2 = axelrod.Player()
        P1.history = ['C','C','C','C']
        P2.history = ['C','C','D','D']
        self.assertEqual(P1.strategy(P2), 'D')

    def test_when_less_than_golder_ratio(self):
        """tests that if the ratio of Cs to Ds is less than the golden ratio then strategy co-operates"""
        P1 = axelrod.Golden()
        P2 = axelrod.Player()
        P1.history = ['C','C','C','C']
        P2.history = ['D','D','D','D']
        self.assertEqual(P1.strategy(P2), 'C')


class TestPi(TestPlayer):

    name = '$\pi$'
    player = axelrod.Pi

    def test_strategy(self):
        """test initial strategy co-operates"""
        P1 = axelrod.Pi()
        P2 = axelrod.Player()
        P2.history = []
        self.assertEqual(P1.strategy(P2), 'C')

    def test_when_no_defection(self):
        """tests that if the opposing player does not defect initially then strategy defects"""
        P1 = axelrod.Pi()
        P2 = axelrod.Player()
        P1.history = ['C']
        P2.history = ['C']
        self.assertEqual(P1.strategy(P2), 'D')

    def test_when_greater_than_pi(self):
        """tests that if the ratio of Cs to Ds is greater than pi then strategy defects"""
        P1 = axelrod.Pi()
        P2 = axelrod.Player()
        P1.history = ['C','C','C','C']
        P2.history = ['C','C','C','D']
        self.assertEqual(P1.strategy(P2), 'D')

    def test_when_less_than_pi(self):
        """tests that if the ratio of Cs to Ds is less than pi then strategy co-operates"""
        P1 = axelrod.Pi()
        P2 = axelrod.Player()
        P1.history = ['C','C','C','C']
        P2.history = ['C','C','D','D']
        self.assertEqual(P1.strategy(P2), 'C')


class e(TestPlayer):

    name = '$e$'
    player = axelrod.e

    def test_strategy(self):
        """test initial strategy co-operates"""
        P1 = axelrod.e()
        P2 = axelrod.Player()
        P2.history = []
        self.assertEqual(P1.strategy(P2), 'C')

    def test_when_no_defection(self):
        """tests that if the opposing player does not defect initially then strategy defects"""
        P1 = axelrod.e()
        P2 = axelrod.Player()
        P1.history = ['C']
        P2.history = ['C']
        self.assertEqual(P1.strategy(P2), 'D')

    def test_when_greater_than_e(self):
        """tests that if the ratio of Cs to Ds is greater than e then strategy defects"""
        P1 = axelrod.e()
        P2 = axelrod.Player()
        P1.history = ['C','C','C','C']
        P2.history = ['C','C','D','D']
        self.assertEqual(P1.strategy(P2), 'D')

    def test_when_less_than_e(self):
        """tests that if the ratio of Cs to Ds is less than e then strategy co-operates"""
        P1 = axelrod.e()
        P2 = axelrod.Player()
        P1.history = ['C','C','C','C']
        P2.history = ['C','D','D','D']
        self.assertEqual(P1.strategy(P2), 'C')



import os
import time
import cPickle as pickle
from tournament import *
from plot import *
from ecosystem import *
import logging
from utils import *


class TournamentManager(object):

    def __init__(self, output_directory, with_ecological,
                 pass_cache=True, load_cache=True, save_cache=False,
                 cache_file='./cache.txt'):
        self.tournaments = []
        self.ecological_variants = []
        self.logger = logging.getLogger(__name__)
        self.output_directory = output_directory
        self.with_ecological = with_ecological
        self.pass_cache = pass_cache
        self.save_cache = save_cache
        self.cache_file = cache_file
        self.deterministic_cache = {}
        self.cache_valid_for_turns = None
        self.load_cache = False

        if load_cache and not save_cache:
            self.load_cache = self.load_cache_from_file(cache_file)

    def one_player_per_strategy(self, strategies):
        return [strategy() for strategy in strategies]

    def add_tournament(self, name, players, game=None, turns=200,
                       repetitions=10, processes=None):
        tournament = Tournament(
            name=name,
            players=players,
            turns=turns,
            repetitions=repetitions,
            processes=processes)
        self.tournaments.append(tournament)

    def run_tournaments(self):
        t0 = time.time()
        for tournament in self.tournaments:
            self.run_single_tournament(tournament)
        if self.save_cache:
            self.save_cache_to_file(self.deterministic_cache, self.cache_file)
        self.logger.debug(timed_message('Finished all tournaments', t0))

    def run_single_tournament(self, tournament):
        self.logger.debug(
            'Starting %s tournament with %d round robins of %d turns per pair.'
            % (tournament.name, tournament.repetitions, tournament.turns))

        t0 = time.time()

        if self.pass_cache and self.valid_cache(tournament.turns):
            self.logger.debug('Passing cache with %d entries to %s tournament' %
                            (len(self.deterministic_cache), tournament.name))
            tournament.deterministic_cache = self.deterministic_cache
            if self.load_cache:
                tournament.prebuilt_cache = True
        else:
            self.logger.debug('Cache is not valid for %s tournament' %
                            tournament.name)
        tournament.play()

        self.logger.debug(timed_message('Finished %s tournament' % tournament.name, t0))

        if self.with_ecological:
            ecosystem = Ecosystem(tournament.result_set)
            self.run_ecological_variant(tournament, ecosystem)
        else:
            ecosystem = None

        self.generate_output_files(tournament, ecosystem)
        self.cache_valid_for_turns = tournament.turns

        self.logger.debug('Cache now has %d entries' %
                        len(self.deterministic_cache))

        self.logger.debug(
            timed_message('Finished all %s tasks' % tournament.name, t0))

    def valid_cache(self, turns):
        return ((len(self.deterministic_cache) == 0) or
                (len(self.deterministic_cache) > 0) and
                turns == self.cache_valid_for_turns)

    def run_ecological_variant(self, tournament, ecosystem):
        self.logger.debug(
            'Starting ecological variant of %s' % tournament.name)
        t0 = time.time()
        ecoturns = {
            'basic_strategies': 100,
            'cheating_strategies': 20,
            'strategies': 200,
            'all_strategies': 40,
        }
        ecosystem.reproduce(ecoturns.get(tournament.name))
        self.logger.debug(
            timed_message('Finished ecological variant of %s' % tournament.name, t0))

    def generate_output_files(self, tournament, ecosystem=None):
        self.save_csv(tournament)
        self.save_plots(tournament, ecosystem)

    def save_csv(self, tournament):
        csv = tournament.result_set.csv()
        file_name = self.output_file_path(
                tournament.name, 'csv')
        with open(file_name, 'w') as f:
            f.write(csv)

    def save_plots(self, tournament, ecosystem=None):
        results = tournament.result_set
        plot = Plot(results)
        if not plot.matplotlib_installed:
            self.logger.debug('The matplotlib library is not installed. '
                            'No plots will be produced')
            return
        for plot_type in ('boxplot', 'payoff'):
            figure = getattr(plot, plot_type)()
            file_name = self.output_file_path(
                tournament.name + '_' + plot_type, 'png')
            self.save_plot(figure, file_name)
        if ecosystem is not None:
            figure = plot.stackplot(ecosystem.population_sizes)
            file_name = self.output_file_path(
                    tournament.name + '_reproduce', 'png')
            self.save_plot(figure, file_name)

    def output_file_path(self, file_name, file_extension):
        return os.path.join(
            self.output_directory,
            file_name + '.' + file_extension)

    def save_plot(self, figure, file_name):
        figure.savefig(file_name, bbox_inches='tight')
        figure.clf()

    def save_cache_to_file(self, cache, file_name):
        self.logger.debug(
            'Saving cache with %d entries to %s' % (len(cache), file_name))
        deterministic_cache = DeterministicCache(
            cache, self.cache_valid_for_turns)
        file = open(file_name, 'w')
        pickle.dump(deterministic_cache, file)
        return True

    def load_cache_from_file(self, file_name):
        try:
            file = open(file_name, 'r')
            deterministic_cache = pickle.load(file)
            self.deterministic_cache = deterministic_cache.cache
            self.cache_valid_for_turns = deterministic_cache.turns
            self.logger.debug(
                'Loaded cache with %d entries' % len(self.deterministic_cache))
            return True
        except IOError:
            self.logger.debug('Cache file not found. Starting with empty cache')
            return False


class DeterministicCache(object):

    def __init__(self, cache, turns):
        self.cache = cache
        self.turns = turns



"""Test for the alternator strategy."""

import axelrod

from test_player import TestPlayer


class TestAlternator(TestPlayer):

    name = "Alternator"
    player = axelrod.Alternator

    def test_strategy(self):
        """Starts by cooperating."""
        P1 = axelrod.Alternator()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_effect_of_strategy(self):
        """Simply does the opposite to what the strategy did last time."""
        P1 = axelrod.Alternator()
        P2 = axelrod.Player()
        P1.history = ['C', 'D', 'D', 'D']
        P2.history = ['C', 'C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history = ['C', 'C', 'D', 'D', 'C']
        P2.history = ['C', 'D', 'C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'D')



from axelrod import Player, RoundRobin, Game
import copy
import inspect

class MindReader(Player):
    """A player that looks ahead at what the opponent will do and decides what to do."""

    name = 'Mind Reader'
    max_look_ahead = 200

    def strategy(self, opponent):
        """Pretends to play the opponent 50 times before each match.
        The primary purpose is to look far enough ahead to see if a defect will be punished by the opponent.
        If the MindReader attempts to play itself (or another similar strategy), then it will cause a recursion loop, so this is also handeled in this method, by defecting if the method is called by strategy
        """

        curframe = inspect.currentframe()
        calframe = inspect.getouterframes(curframe, 2)
        calname = calframe[1][3]

        if calname in ('strategy', 'simulate_match'):
            return 'D'

        best_strategy = self.look_ahead(opponent)

        return best_strategy

    def simulate_match(self, opponent, strategy, rounds = 10):
        """Simulates a number of matches."""
        for match in range(rounds):
            play_1, play_2 = strategy, opponent.strategy(self)
            self.history.append(play_1)
            opponent.history.append(play_2)

    def look_ahead(self, opponent, rounds = 10):
        """Plays a number of rounds to determine the best strategy."""
        results = []
        game = Game()
        round_robin = RoundRobin(players=[self, opponent], game=game, turns=rounds)
        strategies = ['C', 'D']

        dummy_history_self = copy.copy(self.history)
        dummy_history_opponent = copy.copy(opponent.history)

        for strategy in strategies:
            self.simulate_match(opponent, strategy, rounds)
            results.append(round_robin.calculate_scores(self, opponent)[0])

            self.history = copy.copy(dummy_history_self)
            opponent.history = copy.copy(dummy_history_opponent)

        return strategies[results.index(min(results))]

class ProtectedMindReader(MindReader):
    """A player that looks ahead at what the opponent will do and decides what to do.
    It is also protected from mind control strategies"""

    name = 'Protected Mind Reader'

    def __setattr__(self, name, val):
        """Stops any other strategy altering the methods of this class """

        if name == 'strategy':
            pass
        else:
            self.__dict__[name] = val



import unittest
import axelrod

matplotlib_installed = True
try:
    import matplotlib.pyplot
except ImportError:
    matplotlib_installed = False


class TestPlot(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        players = ('Player1', 'Player2', 'Player3')
        test_payoffs_list = [
            [[0, 10, 21], [10, 0, 16], [16, 16, 0]],
            [[0, 10, 21], [8, 0, 20], [16, 16, 0]],
        ]
        cls.test_result_set = axelrod.ResultSet(players, 5, 2)
        cls.test_result_set.finalise(test_payoffs_list)

        cls.expected_boxplot_dataset = [[2.6, 2.8], [3.1, 3.1], [3.2, 3.2]]
        cls.expected_boxplot_xticks_locations = [1, 2, 3, 4]
        cls.expected_boxplot_xticks_labels = ['Player2', 'Player1', 'Player3']
        cls.expected_boxplot_title = ('Mean score per stage game over 5 rounds'
                                      ' repeated 2 times (3 strategies)')

        cls.expected_payoff_dataset = [
            [0.0, 1.8, 3.6],
            [2.0, 0.0, 4.2],
            [3.2, 3.2, 0.0]]

    def test_init(self):
        result_set = axelrod.ResultSet(('Player1', 'Player2', 'Player3'), 5, 2)
        self.assertRaises(AttributeError, axelrod.Plot, result_set)
        plot = axelrod.Plot(self.test_result_set)
        self.assertEqual(plot.result_set, self.test_result_set)
        self.assertEqual(matplotlib_installed, plot.matplotlib_installed)

    def test_boxplot_dataset(self):
        plot = axelrod.Plot(self.test_result_set)
        self.assertSequenceEqual(
            plot.boxplot_dataset(),
            self.expected_boxplot_dataset)

    def test_boxplot_xticks_locations(self):
        plot = axelrod.Plot(self.test_result_set)
        self.assertEqual(
            plot.boxplot_xticks_locations(),
            self.expected_boxplot_xticks_locations)

    def test_boxplot_xticks_labels(self):
        plot = axelrod.Plot(self.test_result_set)
        self.assertEqual(
            plot.boxplot_xticks_labels(),
            self.expected_boxplot_xticks_labels)

    def test_boxplot_title(self):
        plot = axelrod.Plot(self.test_result_set)
        self.assertEqual(plot.boxplot_title(), self.expected_boxplot_title)

    def test_boxplot(self):
        if matplotlib_installed:
            plot = axelrod.Plot(self.test_result_set)
            self.assertIsInstance(plot.boxplot(), matplotlib.pyplot.Figure)
        else:
            self.skipTest('matplotlib not installed')

    def test_payoff_dataset(self):
        plot = axelrod.Plot(self.test_result_set)
        self.assertSequenceEqual(
            plot.payoff_dataset(),
            self.expected_payoff_dataset)

    def test_payoff(self):
        if matplotlib_installed:
            plot = axelrod.Plot(self.test_result_set)
            self.assertIsInstance(plot.payoff(), matplotlib.pyplot.Figure)
        else:
            self.skipTest('matplotlib not installed')

    def test_ecosystem(self):
        if matplotlib_installed:
            eco = axelrod.Ecosystem(self.test_result_set)
            eco.reproduce(100)
            plot = axelrod.Plot(self.test_result_set)
            self.assertIsInstance(plot.stackplot(eco.population_sizes), matplotlib.pyplot.Figure)
        else:
            self.skipTest('matplotlib not installed')


if __name__ == '__main__':
    unittest.main()



"""Test for the once bitten strategy."""

import random

import axelrod

from test_player import TestPlayer


class TestOnceBitten(TestPlayer):

    name = "Once Bitten"
    player = axelrod.OnceBitten
    stochastic = False

    def test_initial_strategy(self):
        """Starts by cooperating."""
        P1 = axelrod.OnceBitten()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_strategy(self):
        """If opponent defects at any point then the player will defect forever."""
        P1 = axelrod.OnceBitten()
        P2 = axelrod.Player()
        # Starts by playing C
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('C')

        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('C')

        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('D')

        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.grudged, False)
        P2.history.append('D')

        self.assertEqual(P2.history, ['C', 'C', 'D', 'D'])
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.grudged, True)

        for turn in range(P1.mem_length-1):
            self.assertEqual(P1.strategy(P2), 'D')
            # Doesn't matter what opponent plays now
            P2.history.append('C')
            self.assertEqual(P1.grudged, True)
            P2.history.append('D')
            self.assertEqual(P1.grudged, True)

        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P1.grudge_memory, 10)
        self.assertEqual(P1.grudged, True)
        P2.history.append('C')

    def test_reset(self):
        """Check that grudged gets reset properly"""
        P1 = self.player()
        P1.history = ['C', 'D']
        P2 = axelrod.Player()
        P2.history = ['D', 'D']
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertTrue(P1.grudged)
        P1.reset()
        self.assertFalse(P1.grudged)
        self.assertEqual(P1.history, [])

class TestFoolMeOnce(TestPlayer):

    name = "Fool Me Once"
    player = axelrod.FoolMeOnce
    stochastic = False

    def test_initial(self):
        P1 = axelrod.FoolMeOnce()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), P1._initial)

    def test_strategy(self):
        """
        If opponent defects at any point then the player will defect forever
        """
        P1 = axelrod.FoolMeOnce()
        P2 = axelrod.Defector()
        P1.history = ['C']
        P2.history = ['D']
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history = ['C']
        P2.history = ['D']
        self.assertEqual(P1.strategy(P2), 'D')


    def test_reset(self):
        """Check that count gets reset properly"""
        P1 = self.player()
        P1.history = ['C', 'D']
        P2 = axelrod.Player()
        P2.history = ['D']
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.D_count, 1)
        P1.reset()
        self.assertEqual(P1.D_count, 0)
        self.assertEqual(P1.history, [])

class TestForgetfulFoolMeOnce(TestPlayer):

    name = 'Forgetful Fool Me Once'
    player = axelrod.ForgetfulFoolMeOnce
    stochastic = True

    def test_initial(self):
        P1 = axelrod.ForgetfulFoolMeOnce()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_strategy(self):
        """Test that will forgive one D but will grudge after 2 Ds, randomly forgets count"""
        random.seed(2)
        P1 = self.player()
        P2 = axelrod.Player()
        P2.history = ['D']
        self.assertEqual(P1.strategy(P2), 'C')
        P2.history = ['D', 'D']
        self.assertEqual(P1.strategy(P2), 'D')
        # Sometime will forget count:
        self.assertEqual(P1.strategy(P2), 'C')

    def test_reset(self):
        """Check that count gets reset properly"""
        P1 = self.player()
        P1.history = ['C', 'D']
        P2 = axelrod.Player()
        P2.history = ['D']
        random.seed(1)
        self.assertEqual(P1.strategy(P2), 'C')
        self.assertEqual(P1.D_count, 1)
        P1.reset()
        self.assertEqual(P1.D_count, 0)
        self.assertEqual(P1.history, [])




from axelrod import Player


class Grudger(Player):
    """A player starts by cooperating however will defect if at any point the opponent has defected."""

    name = 'Grudger'

    def strategy(self, opponent):
        """Begins by playing C, then plays D for the remaining rounds if the opponent ever plays D."""
        if 'D' in opponent.history:
            return 'D'
        return 'C'


class ForgetfulGrudger(Player):
    """A player starts by cooperating however will defect if at any point the opponent has defected, but forgets after meme_length matches."""

    name = 'Forgetful Grudger'

    def __init__(self):
        """Initialised the player."""
        super(ForgetfulGrudger, self).__init__()
        self.history = []
        self.score = 0
        self.mem_length = 10
        self.grudged = False
        self.grudge_memory = 0

    def strategy(self, opponent):
        """Begins by playing C, then plays D for mem_length rounds if the opponent ever plays D."""
        if self.grudge_memory >= self.mem_length:
            self.grudge_memory = 0
            self.grudged = False

        if self.grudged:
            self.grudge_memory += 1
            return 'D'
        elif 'D' in opponent.history[-1:]:
            self.grudged = True
            return 'D'
        return 'C'

    def reset(self):
        """Resets scores and history."""
        self.history = []
        self.grudged = False
        self.grudge_memory = 0


class OppositeGrudger(Player):
    """A player starts by cooperating however will defect if at any point the opponent has defected."""

    name = 'Opposite Grudger'

    def strategy(self, opponent):
        """Begins by playing D, then plays C for the remaining rounds if the opponent ever plays C."""
        if 'C' in opponent.history:
            return 'C'
        return 'D'



# -*- coding: utf-8 -*-

"""
Geller - by Martin Chorley (@martinjc), heavily inspired by Matthew Williams (@voxmjw)

This code is inspired by Matthew Williams' talk
"Cheating at rock-paper-scissors — meta-programming in Python"
given at Django Weekend Cardiff in February 2014.

His code is here: https://github.com/mattjw/rps_metaprogramming
and there's some more info here: http://www.mattjw.net/2014/02/rps-metaprogramming/

This code is **way** simpler than Matt's, as in this exercise we already
have access to the opponent instance, so don't need to go
hunting for it in the stack. Instead we can just call it to
see what it's going to play, and return a result based on that

This is almost certainly cheating, and more than likely against the
spirit of the 'competition' :-)

This code will fall into infinite recursion when played against itself,
a problem that can be alleviated by looking at the name of the function
calling `strategy`. If it is also `strategy`, we should return some
default value - for example 'C', 'D' or random.
"""

import inspect
import random

from axelrod import Player


class Geller(Player):

    name = 'Geller'
    default = lambda self: 'C' if random.random() > 0.5 else 'D'

    def strategy(self, opponent):
        """
        Look at what the opponent will play in the next round and choose a strategy
        that gives the least jail time, which is is equivalent to playing the same
        strategy as that which the opponent will play.
        """
        curframe = inspect.currentframe()
        calframe = inspect.getouterframes(curframe, 2)
        calname = calframe[1][3]
        if calname == 'strategy':
            return self.default()
        else:
            return opponent.strategy(self)

class GellerCooperator(Geller):
    name = 'Geller Cooperator'
    default = lambda self: 'C'

class GellerDefector(Geller):
    name = 'Geller Defector'
    default = lambda self: 'D'



import unittest

import axelrod


class TestGame(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.game = axelrod.Game()

    def test_init(self):
        expected_scores = {('C', 'D'): (5, 0), ('D', 'C'): (0, 5), ('D', 'D'): (4, 4), ('C', 'C'): (2, 2)}
        self.assertEquals(self.game.scores, expected_scores)

    def test_RPTS(self):
        expected_values = (2, 4, 5, 0)
        self.assertEquals(self.game.RPTS(), expected_values)

    def test_score(self):
        self.assertEquals(self.game.score(('C', 'C')), (2, 2))
        self.assertEquals(self.game.score(('D', 'D')), (4, 4))
        self.assertEquals(self.game.score(('C', 'D')), (5, 0))
        self.assertEquals(self.game.score(('D', 'C')), (0, 5))



import inspect


class Player(object):
    """A class for a player in the tournament.

    This is an abstract base class, not intended to be used directly.
    """

    name = "Player"

    def __init__(self):
        """Initiates an empty history and 0 score for a player."""
        self.history = []
        self.stochastic = "random" in inspect.getsource(self.__class__)
        if self.name == "Player":
            self.stochastic = False

    def __repr__(self):
        """The string method for the strategy."""
        return self.name

    def strategy(self, opponent):
        """This is a placeholder strategy."""
        return None

    def play(self, opponent):
        """This pits two players against each other."""
        s1, s2 = self.strategy(opponent), opponent.strategy(self)
        self.history.append(s1)
        opponent.history.append(s2)

    def reset(self):
        """Resets history.

        When creating strategies that create new attributes then this method should be
        re-written (in the inherited class) and should not only reset history but also
        rest all other attributes.
        """
        self.history = []



"""Tests for the Ecosystem class"""

import unittest

import axelrod


class TestEcosystem(unittest.TestCase):



    @classmethod
    def setUpClass(cls):
        cooperators = axelrod.Tournament(players=[
            axelrod.Cooperator(),
            axelrod.Cooperator(),
            axelrod.Cooperator(),
            axelrod.Cooperator(),
        ])
        defector_wins = axelrod.Tournament(players=[
            axelrod.Cooperator(),
            axelrod.Cooperator(),
            axelrod.Cooperator(),
            axelrod.Defector(),
        ])
        cls.res_cooperators = cooperators.play()
        cls.res_defector_wins = defector_wins.play()

    def test_init(self):
        """Are the populations created correctly?"""

        # By default create populations of equal size
        eco = axelrod.Ecosystem(self.res_cooperators)
        pops = eco.population_sizes
        self.assertEquals(eco.nplayers, 4)
        self.assertEquals(len(pops), 1)
        self.assertEquals(len(pops[0]), 4)
        self.assertAlmostEqual(sum(pops[0]), 1.0)
        self.assertEquals(list(set(pops[0])), [0.25])

        # Can pass list of initial population distributions
        eco = axelrod.Ecosystem(self.res_cooperators, population=[.7, .25, .03, .02])
        pops = eco.population_sizes
        self.assertEquals(eco.nplayers, 4)
        self.assertEquals(len(pops), 1)
        self.assertEquals(len(pops[0]), 4)
        self.assertAlmostEqual(sum(pops[0]), 1.0)
        self.assertEquals(pops[0], [.7, .25, .03, .02])

        # Distribution will automatically normalise
        eco = axelrod.Ecosystem(self.res_cooperators, population=[70, 25, 3, 2])
        pops = eco.population_sizes
        self.assertEquals(eco.nplayers, 4)
        self.assertEquals(len(pops), 1)
        self.assertEquals(len(pops[0]), 4)
        self.assertAlmostEqual(sum(pops[0]), 1.0)
        self.assertEquals(pops[0], [.7, .25, .03, .02])

        # If passed list is of incorrect size get error
        self.assertRaises(TypeError, axelrod.Ecosystem, self.res_cooperators, population=[.7, .2, .03, .1, .1])

        # If passed list has negative values
        self.assertRaises(TypeError, axelrod.Ecosystem, self.res_cooperators, population=[.7, -.2, .03, .2])

    def test_cooperators(self):
        """Are cooperators stable over time?"""

        eco = axelrod.Ecosystem(self.res_cooperators)
        eco.reproduce(100)
        pops = eco.population_sizes
        self.assertEquals(len(pops), 101)
        for p in pops:
            self.assertEquals(len(p), 4)
            self.assertEquals(sum(p), 1.0)
            self.assertEquals(list(set(p)), [0.25])

    def test_defector_wins(self):
        """Does one defector win over time?"""

        eco = axelrod.Ecosystem(self.res_defector_wins)
        eco.reproduce(1000)
        pops = eco.population_sizes
        self.assertEquals(len(pops), 1001)
        for p in pops:
            self.assertEquals(len(p), 4)
            self.assertAlmostEquals(sum(p), 1.0)
        last = pops[-1]
        self.assertAlmostEquals(last[0], 0.0)
        self.assertAlmostEquals(last[1], 0.0)
        self.assertAlmostEquals(last[2], 0.0)
        self.assertAlmostEquals(last[3], 1.0)



"""Test for the qlearner strategy."""

import random

import axelrod

from test_player import TestPlayer


class TestRiskyQLearner(TestPlayer):

    name = 'Risky QLearner'
    player = axelrod.RiskyQLearner
    stochastic = True

    def test_qs_update(self):
        """Test that the q and v values update."""
        random.seed(5)
        p1 = axelrod.RiskyQLearner()
        p2 = axelrod.Cooperator()
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.Qs, {'': {'C': 0, 'D': -0.9}, '0.0': {'C': 0, 'D': 0}})
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.Qs,{'': {'C': 0, 'D': -0.9}, '0.0': {'C': 0.9, 'D': 0}, 'C1.0': {'C': 0, 'D': 0}})

    def test_vs_update(self):
        """Test that the q and v values update."""
        random.seed(5)
        p1 = axelrod.RiskyQLearner()
        p2 = axelrod.Cooperator()
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.Vs, {'': 0, '0.0': 0})
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.Vs,{'': 0, '0.0': 0.9, 'C1.0': 0})

    def test_prev_state_updates(self):
        """Test that the q and v values update."""
        random.seed(5)
        p1 = axelrod.RiskyQLearner()
        p2 = axelrod.Cooperator()
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.prev_state, '0.0')
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.prev_state, 'C1.0')

    def test_strategy(self):
        """Tests that it chooses the best strategy."""
        random.seed(5)
        p1 = axelrod.RiskyQLearner()
        p1.state = 'CCDC'
        p1.Qs = {'': {'C': 0, 'D': 0}, 'CCDC': {'C': 2, 'D': 6}}
        p2 = axelrod.Cooperator()
        self.assertEqual(p1.strategy(p2), 'C')
        self.assertEqual(p1.strategy(p2), 'D')
        self.assertEqual(p1.strategy(p2), 'D')
        self.assertEqual(p1.strategy(p2), 'D')
        self.assertEqual(p1.strategy(p2), 'D')
        self.assertEqual(p1.strategy(p2), 'D')

    def test_reset_method(self):
        """
        tests the reset method
        """
        P1 = axelrod.RiskyQLearner()
        P1.Qs = {'': {'C': 0, 'D': -0.9}, '0.0': {'C': 0, 'D': 0}}
        P1.Vs = {'': 0, '0.0': 0}
        P1.history = ['C', 'D', 'D', 'D']
        P1.prev_state = 'C'
        P1.reset()
        self.assertEqual(P1.prev_state, '')
        self.assertEqual(P1.history, [])
        self.assertEqual(P1.Vs, {'':0})
        self.assertEqual(P1.Qs, {'':{'C':0, 'D':0}})


class TestArrogantQLearner(TestPlayer):

    name = 'Arrogant QLearner'
    player = axelrod.ArrogantQLearner
    stochastic = True

    def test_qs_update(self):
        """
        Test that the q and v values update
        """
        random.seed(5)
        p1 = axelrod.ArrogantQLearner()
        p2 = axelrod.Cooperator()
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.Qs, {'': {'C': 0, 'D': -0.9}, '0.0': {'C': 0, 'D': 0}})
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.Qs,{'': {'C': 0, 'D': -0.9}, '0.0': {'C': 0.9, 'D': 0}, 'C1.0': {'C': 0, 'D': 0}})

    def test_vs_update(self):
        """
        Test that the q and v values update
        """
        random.seed(5)
        p1 = axelrod.ArrogantQLearner()
        p2 = axelrod.Cooperator()
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.Vs, {'': 0, '0.0': 0})
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.Vs,{'': 0, '0.0': 0.9, 'C1.0': 0})

    def test_prev_state_updates(self):
        """
        Test that the q and v values update
        """
        random.seed(5)
        p1 = axelrod.ArrogantQLearner()
        p2 = axelrod.Cooperator()
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.prev_state, '0.0')
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.prev_state, 'C1.0')

    def test_strategy(self):
        """Tests that it chooses the best strategy."""
        random.seed(9)
        p1 = axelrod.ArrogantQLearner()
        p1.state = 'CCDC'
        p1.Qs = {'': {'C': 0, 'D': 0}, 'CCDC': {'C': 2, 'D': 6}}
        p2 = axelrod.Cooperator()
        self.assertEqual(p1.strategy(p2), 'C')
        self.assertEqual(p1.strategy(p2), 'D')
        self.assertEqual(p1.strategy(p2), 'D')
        self.assertEqual(p1.strategy(p2), 'D')
        self.assertEqual(p1.strategy(p2), 'D')
        self.assertEqual(p1.strategy(p2), 'D')

    def test_reset_method(self):
        """Tests the reset method."""
        P1 = axelrod.ArrogantQLearner()
        P1.Qs = {'': {'C': 0, 'D': -0.9}, '0.0': {'C': 0, 'D': 0}}
        P1.Vs = {'': 0, '0.0': 0}
        P1.history = ['C', 'D', 'D', 'D']
        P1.prev_state = 'C'
        P1.reset()
        self.assertEqual(P1.prev_state, '')
        self.assertEqual(P1.history, [])
        self.assertEqual(P1.Vs, {'':0})
        self.assertEqual(P1.Qs, {'':{'C':0, 'D':0}})


class TestHesitantQLearner(TestPlayer):

    name = 'Hesitant QLearner'
    player = axelrod.HesitantQLearner
    stochastic = True

    def test_qs_update(self):
        """Test that the q and v values update."""
        random.seed(5)
        p1 = axelrod.HesitantQLearner()
        p2 = axelrod.Cooperator()
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.Qs, {'': {'C': 0, 'D': -0.1}, '0.0': {'C': 0, 'D': 0}})
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.Qs,{'': {'C': 0, 'D': -0.1}, '0.0': {'C': 0.1, 'D': 0}, 'C1.0': {'C': 0, 'D': 0}})

    def test_vs_update(self):
        """
        Test that the q and v values update
        """
        random.seed(5)
        p1 = axelrod.HesitantQLearner()
        p2 = axelrod.Cooperator()
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.Vs, {'': 0, '0.0': 0})
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.Vs,{'': 0, '0.0': 0.1, 'C1.0': 0})

    def test_prev_state_updates(self):
        """
        Test that the q and v values update
        """
        random.seed(5)
        p1 = axelrod.HesitantQLearner()
        p2 = axelrod.Cooperator()
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.prev_state, '0.0')
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.prev_state, 'C1.0')

    def test_strategy(self):
        """Tests that it chooses the best strategy."""
        random.seed(9)
        p1 = axelrod.HesitantQLearner()
        p1.state = 'CCDC'
        p1.Qs = {'': {'C': 0, 'D': 0}, 'CCDC': {'C': 2, 'D': 6}}
        p2 = axelrod.Cooperator()
        self.assertEqual(p1.strategy(p2), 'C')
        self.assertEqual(p1.strategy(p2), 'D')
        self.assertEqual(p1.strategy(p2), 'D')
        self.assertEqual(p1.strategy(p2), 'D')
        self.assertEqual(p1.strategy(p2), 'D')
        self.assertEqual(p1.strategy(p2), 'D')

    def test_reset_method(self):
        """
        tests the reset method
        """
        P1 = axelrod.HesitantQLearner()
        P1.Qs = {'': {'C': 0, 'D': -0.9}, '0.0': {'C': 0, 'D': 0}}
        P1.Vs = {'': 0, '0.0': 0}
        P1.history = ['C', 'D', 'D', 'D']
        P1.prev_state = 'C'
        P1.reset()
        self.assertEqual(P1.prev_state, '')
        self.assertEqual(P1.history, [])
        self.assertEqual(P1.Vs, {'':0})
        self.assertEqual(P1.Qs, {'':{'C':0, 'D':0}})


class TestCautiousQLearner(TestPlayer):

    name = 'Cautious QLearner'
    player = axelrod.CautiousQLearner
    stochastic = True

    def test_qs_update(self):
        """Test that the q and v values update."""
        random.seed(5)
        p1 = axelrod.CautiousQLearner()
        p2 = axelrod.Cooperator()
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.Qs, {'': {'C': 0, 'D': -0.1}, '0.0': {'C': 0, 'D': 0}})
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.Qs,{'': {'C': 0, 'D': -0.1}, '0.0': {'C': 0.1, 'D': 0}, 'C1.0': {'C': 0, 'D': 0}})

    def test_vs_update(self):
        """Test that the q and v values update."""
        random.seed(5)
        p1 = axelrod.CautiousQLearner()
        p2 = axelrod.Cooperator()
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.Vs, {'': 0, '0.0': 0})
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.Vs,{'': 0, '0.0': 0.1, 'C1.0': 0})

    def test_prev_state_updates(self):
        """Test that the q and v values update."""
        random.seed(5)
        p1 = axelrod.CautiousQLearner()
        p2 = axelrod.Cooperator()
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.prev_state, '0.0')
        play_1, play_2 = p1.strategy(p2), p2.strategy(p1)
        p1.history.append(play_1)
        p2.history.append(play_2)
        self.assertEqual(p1.prev_state, 'C1.0')

    def test_strategy(self):
        """Tests that it chooses the best strategy."""
        random.seed(9)
        p1 = axelrod.CautiousQLearner()
        p1.state = 'CCDC'
        p1.Qs = {'': {'C': 0, 'D': 0}, 'CCDC': {'C': 2, 'D': 6}}
        p2 = axelrod.Cooperator()
        self.assertEqual(p1.strategy(p2), 'C')
        self.assertEqual(p1.strategy(p2), 'D')
        self.assertEqual(p1.strategy(p2), 'D')
        self.assertEqual(p1.strategy(p2), 'D')
        self.assertEqual(p1.strategy(p2), 'D')
        self.assertEqual(p1.strategy(p2), 'D')

    def test_reset_method(self):
        """Tests the reset method."""
        P1 = axelrod.CautiousQLearner()
        P1.Qs = {'': {'C': 0, 'D': -0.9}, '0.0': {'C': 0, 'D': 0}}
        P1.Vs = {'': 0, '0.0': 0}
        P1.history = ['C', 'D', 'D', 'D']
        P1.prev_state = 'C'
        P1.reset()
        self.assertEqual(P1.prev_state, '')
        self.assertEqual(P1.history, [])
        self.assertEqual(P1.Vs, {'':0})
        self.assertEqual(P1.Qs, {'':{'C':0, 'D':0}})



"""Test for the inverse strategy."""

import random

import axelrod

from test_player import TestPlayer


class TestInverse(TestPlayer):

    name = "Inverse"
    player = axelrod.Inverse
    stochastic = True

    def test_strategy(self):
        """
        Test that initial strategy cooperates.
        """
        P1 = axelrod.Inverse()
        P2 = axelrod.Player()
        P2.history = []
        self.assertEqual(P1.strategy(P2), 'C')

    def test_that_cooperate_if_opponent_has_not_defected(self):
        """
        Test that as long as the opponent has not defected the player will cooperate.
        """
        P1 = axelrod.Inverse()
        P2 = axelrod.Player()
        P2.history = ['C', 'C', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'C')
        P2.history.append('C')
        self.assertEqual(P1.strategy(P2), 'C')

    def test_when_opponent_has_all_Ds(self):
        """
        Tests that if opponent has played all D then player chooses D
        """
        random.seed(5)
        P1 = axelrod.Inverse()
        P2 = axelrod.Player()
        P2.history = ['D']
        self.assertEqual(P1.strategy(P2), 'D')
        P2.history = ['D', 'D']
        self.assertEqual(P1.strategy(P2), 'D')
        P2.history = ['D', 'D', 'D', 'D', 'D', 'D', 'D', 'D']
        self.assertEqual(P1.strategy(P2), 'D')

    def test_when_opponent_som_Ds(self):
        """
        Tests that if opponent has played all D then player chooses D
        """
        random.seed(5)
        P1 = axelrod.Inverse()
        P2 = axelrod.Player()
        P2.history = ['C', 'D', 'C', 'D']
        self.assertEqual(P1.strategy(P2), 'C')
        P2.history = ['C', 'C', 'C', 'C', 'D', 'D']
        self.assertEqual(P1.strategy(P2), 'C')
        P2.history = ['D', 'D', 'D', 'D', 'D', 'D', 'D', 'D', 'C']
        self.assertEqual(P1.strategy(P2), 'D')
        random.seed(5)
        self.assertEqual(P1.strategy(P2), 'D')



matplotlib_installed = True
try:
    import matplotlib.pyplot as plt
    import matplotlib.transforms as transforms
except ImportError:
    matplotlib_installed = False


class Plot(object):

    def __init__(self, result_set):
        if result_set.finalised:
            self.result_set = result_set
        else:
            raise AttributeError("Result Set is not finalised")
        self.nplayers = self.result_set.nplayers
        self.matplotlib_installed = matplotlib_installed

    def boxplot_dataset(self):
        return [self.result_set.scores[ir] for ir in self.result_set.ranking]

    def payoff_dataset(self):
        return [[self.result_set.payoff_matrix[r1][r2]
                for r2 in self.result_set.ranking]
                for r1 in self.result_set.ranking]

    def boxplot_xticks_locations(self):
        return range(1, len(self.result_set.ranked_names) + 2)

    def boxplot_xticks_labels(self):
        return [str(n) for n in self.result_set.ranked_names]

    def boxplot_title(self):
        return ("Mean score per stage game over {} "
                "rounds repeated {} times ({} strategies)").format(
            self.result_set.turns,
            self.result_set.repetitions,
            len(self.result_set.ranking))

    def boxplot(self):

        if not self.matplotlib_installed:
            return None

        figure = plt.figure()
        plt.boxplot(self.boxplot_dataset())
        plt.xticks(
            self.boxplot_xticks_locations(),
            self.boxplot_xticks_labels(),
            rotation=90)
        plt.tick_params(axis='both', which='both', labelsize=8)
        plt.title(self.boxplot_title())
        return figure

    def payoff(self):

        if not self.matplotlib_installed:
            return None

        figure, ax = plt.subplots()
        mat = ax.matshow(self.payoff_dataset())
        plt.xticks(range(self.result_set.nplayers))
        plt.yticks(range(self.result_set.nplayers))
        ax.set_xticklabels(self.result_set.ranked_names, rotation=90)
        ax.set_yticklabels(self.result_set.ranked_names)
        plt.tick_params(axis='both', which='both', labelsize=8)
        figure.colorbar(mat)
        return figure

    def stackplot(self, populations):

        if not self.matplotlib_installed:
            return None

        figure, ax = plt.subplots()
        turns = range(len(populations))
        pops = [[populations[iturn][ir] for iturn in turns] for ir in self.result_set.ranking]
        ax.stackplot(turns, *pops)

        ax.yaxis.tick_right()
        ax.yaxis.set_label_position("right")
        ax.yaxis.labelpad = 25.0

        plt.ylim([0.0, 1.0])
        plt.ylabel('Relative population size')
        plt.xlabel('Turn')
        plt.title("Strategy population dynamics based on average payoffs")

        ax2 = ax.twinx()
        trans = transforms.blended_transform_factory(ax.transAxes, ax.transData)
        ticks = []
        for i, n in enumerate(self.result_set.ranked_names):
            x = -0.02
            y = (i + 0.5) * 1.0 / self.nplayers
            ax.annotate(n, xy=(x, y), xycoords=trans, clip_on=False, va='center', ha='right', fontsize=8)
            ticks.append(y)
        ax.set_yticks(ticks)
        ax.tick_params(direction='out')
        ax.set_yticklabels([])

        return figure



"""Test for the appeaser strategy."""

import axelrod

from test_player import TestPlayer


class TestAppeaser(TestPlayer):

    name = "Appeaser"
    player = axelrod.Appeaser

    def test_strategy(self):
        P1 = axelrod.Appeaser()
        P2 = axelrod.Player()
        P1.str = 'C';
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history = ['C']
        P2.history = ['C']
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history = ['C', 'D', 'C']
        P2.history = ['C', 'C', 'D']
        self.assertEqual(P1.strategy(P2), 'D')



from axelrod import Player
from math import sqrt, pi, e

class CotoDeRatio(Player):
    """The player will always aim to bring the ratio of co-operations to defections closer to the ratio as given in a sub class"""

    def strategy(self,opponent):

        """initially co-operates"""
        if len(opponent.history) == 0:
            return 'C'

        """to avoid initial division by zero"""
        if sum([s == 'D' for s in opponent.history]) == 0:
            return 'D'

        """otherwise compare ratio to golden mean"""
        if (sum([p == 'C' for p in opponent.history]) + sum([q == 'C' for q in self.history]))/(sum([x == 'D' for x in opponent.history]) + sum([y == 'D' for y in self.history])) > self.ratio:
            return 'D'
        return 'C'


class Golden(CotoDeRatio):
    """The player will always aim to bring the ratio of co-operations to defections closer to the golden mean"""

    name = '$\phi$'
    ratio = (1 + sqrt(5)) / 2


class Pi(CotoDeRatio):
    """The player will always aim to bring the ratio of co-operations to defections closer to the pi"""

    name = '$\pi$'
    ratio = pi


class e(CotoDeRatio):
    """The player will always aim to bring the ratio of co-operations to defections closer to the e"""

    name = '$e$'
    ratio = e



from distutils.core import setup

setup(
    name='Axelrod',
    version='0.0.2',
    author='Vince Knight',
    author_email='vincent.knight@gmail.com',
    packages=['axelrod', 'axelrod.strategies', 'axelrod.tests'],
    scripts=['run_tournament.py', 'README.rst'],
    url='http://axelrod.readthedocs.org/',
    license='The MIT License (MIT)',
    description='Reproduce the Axelrod iterated prisoners dilemma tournament',
    long_description=open('README.rst').read(),
    install_requires=[
        "matplotlib >= 1.4.2",
    ],
)



from axelrod import Player


class Cooperator(Player):
    """A player who only ever cooperates."""

    name = 'Cooperator'

    def strategy(self, opponent):
        return 'C'


class TrickyCooperator(Player):
    """A cooperator that is trying to be tricky."""

    name = "Tricky Cooperator"

    def strategy(self, opponent):
        """Almost always cooperates, but will try to trick the opponent by defecting.

        Defect once in a while in order to get a better payout, when the opponent
        has not defected in the last ten turns and only cooperated during last 3 turns.
        """
        if 'D' not in opponent.history[-10:] and opponent.history[-3:] == ['C']*3:
            return 'D'
        return 'C'
        